@@ -83,6 +83,7 @@ static char rcsid[] = "$OpenBSD: ar.c,v 1.3 1997/01/15 23:42:11 millert Exp $";
 #include "archive.h"
 #include "extern.h"
 #include "stuff/execute.h"
+#include "stuff/unix_standard_mode.h"
 
 CHDR chdr;
 u_int options;
@@ -102,13 +103,15 @@ main(argc, argv)
 	int argc;
 	char **argv;
 {
-	int c, retval, verbose;
+	int c, retval, verbose, run_ranlib, toc64;
 	char *p;
 	int (*fcall) __P((char **));
 
 	fcall = 0;
 	verbose = 0;
+	toc64 = 0;
 	progname = argv[0];
+	run_ranlib = 1;
 
 	if (argc < 3)
 		usage();
@@ -131,7 +134,7 @@ main(argc, argv)
 	 * extended format #1.  The new option -L allows ar to use the extended 
 	 * format and the old -T option causes the truncation of names.
 	 */
-	while ((c = getopt(argc, argv, "abcdilLmopqrsTtuVvx")) != -1) {
+	while ((c = getopt(argc, argv, "abcdilLmopqrSsTtuVvx6")) != -1) {
 		switch(c) {
 		case 'a':
 			options |= AR_A;
@@ -175,6 +178,10 @@ main(argc, argv)
 		case 's':
 			options |= AR_S;
 			break;
+		case 'S':
+			options &= ~AR_S;
+			run_ranlib = 0;
+			break;
 		case 'T':
 			options |= AR_TR;
 			break;
@@ -188,6 +195,9 @@ main(argc, argv)
 		case 'V':
 			verbose = 1;
 			break;
+		case '6':
+			toc64 = 1;
+			break;
 		case 'v':
 			options |= AR_V;
 			break;
@@ -256,19 +266,33 @@ main(argc, argv)
 
 	if(fcall != 0){
 	    retval = (*fcall)(argv);
-	    if(retval != EXIT_SUCCESS || (options & AR_S) != AR_S)
+	    if(retval != EXIT_SUCCESS ||
+	       ((options & AR_S) != AR_S &&
+	        (get_unix_standard_mode() == FALSE ||
+		 archive_opened_for_writing == 0)))
 		exit(retval);
 	}
 
-	/* run ranlib -f on the archive */
-	reset_execute_list();
-	add_execute_list("ranlib");
-	add_execute_list("-f");
-	add_execute_list(archive);
-	if(execute_list(verbose) == 0){
-	    (void)fprintf(stderr, "%s: internal ranlib command failed\n",
-			  progname);
-	    exit(EXIT_FAILURE);
+	/*
+	 * The default is to run ranlib(1) for UNIX conformance.  But if the -S
+	 * option is specified by the user we don't run it.
+	 */
+	if(run_ranlib){
+	    /* run ranlib -f or -q on the archive */
+	    reset_execute_list();
+	    add_execute_list_with_prefix("ranlib");
+	    if(options & AR_S)
+		add_execute_list("-f");
+	    else
+		add_execute_list("-q");
+	    if(toc64)
+		add_execute_list("-toc64");
+	    add_execute_list(archive);
+	    if(execute_list(verbose) == 0){
+		(void)fprintf(stderr, "%s: internal ranlib command failed\n",
+			      progname);
+		exit(EXIT_FAILURE);
+	    }
 	}
 	exit(EXIT_SUCCESS);
 }
