@@ -36,6 +36,9 @@ char *scrub_last_string = NULL;
 #ifdef NeXT_MOD	/* .include feature */
 /* These are moved out of do_scrub() so save_scrub_context() can save them */
 static int state;
+#ifdef I386
+static int substate = 0;
+#endif
 static int old_state;
 static char *out_string;
 static char out_buf[20];
@@ -67,6 +70,7 @@ void)
 	memset(lex, '\0', sizeof(lex));		/* Trust NOBODY! */
 	lex [' ']		|= LEX_IS_WHITESPACE;
 	lex ['\t']		|= LEX_IS_WHITESPACE;
+	lex ['\r']		|= LEX_IS_WHITESPACE;
 	for (p =symbol_chars;*p;++p)
 		lex [(int)*p] |= LEX_IS_SYMBOL_COMPONENT;
 	lex ['\n']		|= LEX_IS_LINE_SEPERATOR;
@@ -127,10 +131,21 @@ FILE *fp)
 		6: putting out \ escape in a "d string.
 		7: After putting out a .file, put out string.
 		8: After putting out a .file string, flush until newline.
+	        FROM line 358
+		9: After seeing symbol char in state 3 (keep 1white after symchar)
+	       10: After seeing whitespace in state 9 (keep white before symchar)
 		-1: output string in out_string and go to the state in old_state
 		-2: flush text until a '*' '/' is seen, then go to state old_state
 	*/
 
+			       
+  /* FROM line 379 */
+  /* I added states 9 and 10 because the MIPS ECOFF assembler uses
+     constructs like ``.loc 1 20''.  This was turning into ``.loc
+     120''.  States 9 and 10 ensure that a space is never dropped in
+     between characters which could appear in an identifier.  Ian
+     Taylor, ian@cygnus.com.  */
+
 #ifndef NeXT_MOD	/* .include feature */
 	static state;
 	static old_state;
@@ -275,6 +290,9 @@ FILE *fp)
 		do ch= getc_unlocked(fp);
 		while(ch!='\n');
 		state=0;
+#ifdef I386
+		substate = 0;
+#endif
 		return ch;
 	}
 
@@ -293,6 +311,18 @@ FILE *fp)
 		}
 		ungetc(ch, fp);
 		if(state==0 || state==2) {
+#ifdef I386
+			if(state == 2){
+			    if(substate == 0){
+				/* if in state 2 don't change to state 3
+				   the first time, and leave white space
+				   after the first two tokens */
+				substate = 1;
+				return ' ';
+			    }
+			    substate = 0;
+			}
+#endif
 			state++;
 			return ' ';
 		}
@@ -301,7 +331,10 @@ FILE *fp)
 			return ' ';
 		}
 #endif
-		else goto flushchar;
+		/* FROM line 900 */
+		if (state == 9)
+		  state = 10;
+		goto flushchar;
 
 	case '/':
 		ch=getc_unlocked(fp);
@@ -374,8 +407,12 @@ FILE *fp)
 		return *out_string++;
 
 	case ':':
-		if(state!=3)
+		if(state!=3) {
 			state=0;
+#ifdef I386
+			substate = 0;
+#endif
+		}
 		return ch;
 
 	case '\n':
@@ -390,10 +427,40 @@ FILE *fp)
 	case ';':
 #endif
 		state=0;
+#ifdef I386
+		substate = 0;
+#endif
 		return ch;
 
 	default:
 	deal_misc:
+		/* FROM line 1234 */
+		if (IS_SYMBOL_COMPONENT (ch))
+		  {
+		    if (state == 10)
+		      {
+			/* This is a symbol character following another symbol
+			   character, with whitespace in between.  We skipped
+			   the whitespace earlier, so output it now.  */
+			ungetc(ch, fp);
+			state = 3;
+			ch = ' ';
+			return ch;
+		      }
+
+		    if (state == 3)
+		      state = 9;
+		  }
+		/* FROM line 1321 */
+		else if (state == 9)
+		  {
+		    /* FROM line 1324 */
+		    state = 3;
+		  }
+		else if (state == 10)
+		  /* FROM line 1345 */
+		  state = 3;
+
 		if(state==0 && IS_LINE_COMMENT(ch)) {
 			do ch=getc_unlocked(fp);
 			while(ch!=EOF && IS_WHITESPACE(ch));
@@ -409,6 +476,9 @@ FILE *fp)
 				if(ch==EOF)
 					as_warn("EOF in Comment: Newline inserted");
 				state=0;
+#ifdef I386
+				substate = 0;
+#endif
 				return '\n';
 			}
 			ungetc(ch, fp);
@@ -423,6 +493,9 @@ FILE *fp)
 			if(ch==EOF)
 				as_warn("EOF in comment:  Newline inserted");
 			state=0;
+#ifdef I386
+			substate = 0;
+#endif
 			return '\n';
 
 		} else if(state==0) {
@@ -444,7 +517,7 @@ FILE *fp)
 }
 
 int
-do_scrub_next_char_from_string()
+do_scrub_next_char_from_string(void)
 {
 	/* State 0: beginning of normal line
 		1: After first whitespace on normal line (flush more white)
@@ -602,6 +675,9 @@ do_scrub_next_char_from_string()
 		do ch= scrub_from_string();
 		while(ch!='\n');
 		state=0;
+#ifdef I386
+		substate = 0;
+#endif
 		return ch;
 	}
 
@@ -701,8 +777,12 @@ do_scrub_next_char_from_string()
 		return *out_string++;
 
 	case ':':
-		if(state!=3)
+		if(state!=3) {
 			state=0;
+#ifdef I386
+			substate = 0;
+#endif
+		}
 		return ch;
 
 	case '\n':
@@ -717,6 +797,9 @@ do_scrub_next_char_from_string()
 	case ';':
 #endif
 		state=0;
+#ifdef I386
+		substate = 0;
+#endif
 		return ch;
 
 	default:
@@ -736,6 +819,9 @@ do_scrub_next_char_from_string()
 				if(ch==EOF)
 					as_warn("EOF in Comment: Newline inserted");
 				state=0;
+#ifdef I386
+				substate = 0;
+#endif
 				return '\n';
 			}
 			scrub_to_string(ch);
@@ -750,6 +836,9 @@ do_scrub_next_char_from_string()
 			if(ch==EOF)
 				as_warn("EOF in comment:  Newline inserted");
 			state=0;
+#ifdef I386
+	    		substate = 0;
+#endif
 			return '\n';
 
 		} else if(state==0) {
