@@ -58,7 +58,7 @@ static int i860_ip(
     char *str);
 static void md_insn_to_chars(
     unsigned char *buf,
-    long val,
+    int32_t val,
     int n);
 
 const relax_typeS md_relax_table[] = { {0} };
@@ -67,11 +67,11 @@ const relax_typeS md_relax_table[] = { {0} };
 static struct hash_control *op_hash = NULL;
 
 static void s_dual(
-    int mode);
+    uintptr_t mode);
 static void s_i860_align(
-    int value);
+    uintptr_t value);
 static void s_i860_org(
-    int value);
+    uintptr_t value);
 
 const pseudo_typeS md_pseudo_table[] = {
     { "float",	float_cons,	'f'	},
@@ -130,11 +130,11 @@ static int insn_count;	/* Track insns assembled, as a word count */
 
 struct i860_it {
     char    *error;
-    unsigned long opcode;
-    struct nlist *nlistp;
+    uint32_t opcode;
+    nlist_t *nlistp;
     expressionS exp;
     int pcrel;
-    enum reloc_type_i860 reloc;
+    int reloc;
 };
 static struct i860_it the_insn;
 
@@ -153,7 +153,7 @@ static char *expr_end;
 static
 void
 s_dual(
-int mode)
+uintptr_t mode)
 {
 	dual_insn_mode = mode;
 }
@@ -161,10 +161,10 @@ int mode)
 static
 void
 s_i860_align(
-int value)
+uintptr_t value)
 {
     register unsigned int temp;
-    register long int temp_fill;
+    register int32_t temp_fill;
     unsigned int i = 0;
     unsigned int bytes;
     char *toP, fill;
@@ -231,11 +231,11 @@ int value)
 static
 void
 s_i860_org(
-int value)
+uintptr_t value)
 {
 	register segT segment;
 	expressionS exp;
-	register long int temp_fill;
+	register int32_t temp_fill;
 	register char *p;
 	extern segT get_known_segmented_expression();
 
@@ -268,8 +268,8 @@ int value)
 	else if ( exp.X_add_number & 3 )
 	{
 	    exp.X_add_number &= ~3;
-	    as_warn(".org not on instruction boundry. Adjusted to \".org %ld\"",
-		exp.X_add_number);
+	    as_warn(".org not on instruction boundry. Adjusted to \".org "
+		    "%lld\"", exp.X_add_number);
 	}
 	if ( exp.X_add_symbol == NULL )
 		insn_count = exp.X_add_number >> 2;
@@ -289,8 +289,8 @@ void
 md_begin(
 void)
 {
-    register char *retval = NULL;
-    unsigned long i;
+    const char *retval = NULL;
+    uint32_t i;
     int j = 0;
 
     insn_count = 0;
@@ -393,7 +393,7 @@ char *str)
     struct i860_opcode *insn;
     char *argsStart;
     char *s1;
-    unsigned long   opcode;
+    uint32_t   opcode;
     unsigned int mask;
     int this_insn_is_dual = 0;
     int adjustment;
@@ -775,14 +775,14 @@ char *str)
 		    	if ( IS_LOGOP(opcode) )
 			{
 			    if ( ((unsigned)the_insn.exp.X_add_number) > 0xFFFF )
-			    	as_warn("%lu is too big for 16 bit unsigned value!",
+			    	as_warn("%lld is too big for 16 bit unsigned value!",
 					the_insn.exp.X_add_number);
 			}
 			else
 			{
 			    if ( ((int)the_insn.exp.X_add_number) > 32767 ||
 			    	 ((int)the_insn.exp.X_add_number) < -32768 )
-				as_warn("%ld is out of range for 16 bit signed value!",
+				as_warn("%lld is out of range for 16 bit signed value!",
 					the_insn.exp.X_add_number);
 					
 			    if ((align_mask & the_insn.exp.X_add_number) != 0)
@@ -941,7 +941,7 @@ int *sizeP)
 	input_line_pointer=t;
     *sizeP=prec * sizeof(LITTLENUM_TYPE);
     for(wordP=words;prec--;) {
-	md_number_to_chars(litP,(long)(*wordP++),sizeof(LITTLENUM_TYPE));
+	md_number_to_chars(litP,(int32_t)(*wordP++),sizeof(LITTLENUM_TYPE));
 	litP+=sizeof(LITTLENUM_TYPE);
     }
     return "";	/* Someone should teach Dean about null pointers */
@@ -953,7 +953,7 @@ int *sizeP)
 void
 md_number_to_chars(
 char *buf,
-long val,
+signed_expr_t val,
 int n)
 {
 
@@ -983,7 +983,7 @@ static
 void
 md_insn_to_chars(
 unsigned char *buf,
-long val,
+int32_t val,
 int n)
 {
 
@@ -1014,7 +1014,7 @@ static
 void
 md_insn_to_chars(
 unsigned char *buf,
-long val,
+int32_t val,
 int n)
 {
 	md_number_to_chars((char *)buf,val,n);
@@ -1024,12 +1024,12 @@ int n)
 void
 md_number_to_imm(
 unsigned char *buf,
-long val,
+signed_expr_t val,
 int n,
 fixS *fixP,
 int nsect)
 {
-    unsigned long opcode;
+    uint32_t opcode;
 
     if ( nsect == (int)text_nsect && (n % 4) != 0 )
     	as_warn("Immediate write of non-aligned data into text segment." );
