@@ -19,6 +19,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include "as.h"
 #include "hash.h"
 #include "obstack.h"		/* For "symbols.h" */
@@ -32,6 +33,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "messages.h"
 #include "fixes.h"
 #include "input-scrub.h"
+#include "dwarf2dbg.h"
 
 /* symbol-name => struct symbol pointer */
 struct hash_control *sy_hash = NULL;
@@ -69,16 +71,14 @@ symbolS	abs_symbol = { {{0}} };
 
 typedef short unsigned int local_label_countT;
 
-static local_label_countT local_label_counter[10];
-
-static				/* Returned to caller, then copied. */
-  char symbol_name_build[12];	/* used for created names ("4f") */
-
-static long int symbol_count = 0;	/* The number of symbols we've declared. */
-
 static void make_stab_for_symbol(
     symbolS *symbolP);
 
+static void make_subprogram_for_symbol(
+    symbolS *symbolP);
+
+static void fb_label_init(void);
+
 
 void
 symbol_begin(
@@ -89,56 +89,225 @@ void)
   sy_hash = hash_new();
   memset((char *)(&abs_symbol), '\0', sizeof(abs_symbol));
   abs_symbol.sy_type = N_ABS;	/* Can't initialise a union. Sigh. */
-  memset((char *)(local_label_counter), '\0', sizeof(local_label_counter) );
+  fb_label_init ();
 }
 
-/*
- *			local_label_name()
- *
- * Caller must copy returned name: we re-use the area for the next name.
- */
-char *				/* Return local label name. */
-local_label_name(
-int n,		/* we just saw "n:", "nf" or "nb" : n a digit */
-int augend)	/* 0 for nb, 1 for n:, nf */
-{
-  register char *	p;
-  register char *	q;
-  char symbol_name_temporary[10]; /* build up a number, BACKWARDS */
 
-  know( n >= 0 );
-  know( augend == 0 || augend == 1 );
+/* Somebody else's idea of local labels. They are made by "n:" where n
+   is any decimal digit. Refer to them with
+    "nb" for previous (backward) n:
+   or "nf" for next (forward) n:.
+
+   We do a little better and let n be any number, not just a single digit, but
+   since the other guy's assembler only does ten, we treat the first ten
+   specially.
+
+   Like someone else's assembler, we have one set of local label counters for
+   entire assembly, not one set per (sub)segment like in most assemblers. This
+   implies that one can refer to a label in another segment, and indeed some
+   crufty compilers have done just that.
+
+   Since there could be a LOT of these things, treat them as a sparse
+   array.  */
+
+#define LOCAL_LABEL_CHAR	'\002'
+#define FB_LABEL_SPECIAL (10)
+
+static int32_t fb_low_counter[FB_LABEL_SPECIAL];
+static int32_t *fb_labels;
+static int32_t *fb_label_instances;
+static int32_t fb_label_count;
+static int32_t fb_label_max;
+
+/* This must be more than FB_LABEL_SPECIAL.  */
+#define FB_LABEL_BUMP_BY (FB_LABEL_SPECIAL + 6)
+
+static void
+fb_label_init (void)
+{
+  memset ((void *) fb_low_counter, '\0', sizeof (fb_low_counter));
+}
+
+/* Add one to the instance number of this fb label.  */
+
+void
+fb_label_instance_inc (int32_t label)
+{
+  int32_t *i;
+
+  if (label < FB_LABEL_SPECIAL)
+    {
+      ++fb_low_counter[label];
+      return;
+    }
+
+  if (fb_labels != NULL)
+    {
+      for (i = fb_labels + FB_LABEL_SPECIAL;
+	   i < fb_labels + fb_label_count; ++i)
+	{
+	  if (*i == label)
+	    {
+	      ++fb_label_instances[i - fb_labels];
+	      return;
+	    }			/* if we find it  */
+	}			/* for each existing label  */
+    }
+
+  /* If we get to here, we don't have label listed yet.  */
+
+  if (fb_labels == NULL)
+    {
+      fb_labels = (int32_t *) xmalloc (FB_LABEL_BUMP_BY * sizeof (int32_t));
+      fb_label_instances = (int32_t *) xmalloc (FB_LABEL_BUMP_BY * sizeof (int32_t));
+      fb_label_max = FB_LABEL_BUMP_BY;
+      fb_label_count = FB_LABEL_SPECIAL;
+
+    }
+  else if (fb_label_count == fb_label_max)
+    {
+      fb_label_max += FB_LABEL_BUMP_BY;
+      fb_labels = (int32_t *) xrealloc ((char *) fb_labels,
+				     fb_label_max * sizeof (int32_t));
+      fb_label_instances = (int32_t *) xrealloc ((char *) fb_label_instances,
+					      fb_label_max * sizeof (int32_t));
+    }				/* if we needed to grow  */
+
+  fb_labels[fb_label_count] = label;
+  fb_label_instances[fb_label_count] = 1;
+  ++fb_label_count;
+}
+
+static int32_t
+fb_label_instance (int32_t label)
+{
+  int32_t *i;
+
+  if (label < FB_LABEL_SPECIAL)
+    {
+      return (fb_low_counter[label]);
+    }
+
+  if (fb_labels != NULL)
+    {
+      for (i = fb_labels + FB_LABEL_SPECIAL;
+	   i < fb_labels + fb_label_count; ++i)
+	{
+	  if (*i == label)
+	    {
+	      return (fb_label_instances[i - fb_labels]);
+	    }			/* if we find it  */
+	}			/* for each existing label  */
+    }
+
+  /* We didn't find the label, so this must be a reference to the
+     first instance.  */
+  return 0;
+}
+
+/* Caller must copy returned name: we re-use the area for the next name.
+
+   The mth occurence of label n: is turned into the symbol "Ln^Bm"
+   where n is the label number and m is the instance number. "L" makes
+   it a label discarded unless debugging and "^B"('\2') ensures no
+   ordinary symbol SHOULD get the same name as a local label
+   symbol. The first "4:" is "L4^B1" - the m numbers begin at 1. */
+
+char *				/* Return local label name.  */
+fb_label_name (int32_t n,	/* We just saw "n:", "nf" or "nb" : n a number.  */
+	       int32_t augend	/* 0 for nb, 1 for n:, nf.  */)
+{
+  int32_t i;
+  /* Returned to caller, then copied.  Used for created names ("4f").  */
+  static char symbol_name_build[24];
+  register char *p;
+  register char *q;
+  char symbol_name_temporary[20];	/* Build up a number, BACKWARDS.  */
+
+  know (n >= 0);
+#ifdef TC_MMIX
+  know ((uint32_t) augend <= 2 /* See mmix_fb_label.  */);
+#else
+  know ((uint32_t) augend <= 1);
+#endif
   p = symbol_name_build;
-  * p ++ = 'L';
-  * p ++ = n + '0';		/* Make into ASCII */
-  * p ++ = 1;			/* ^A */
-  n = local_label_counter [ n ] + augend;
-				/* version number of this local label */
-  /*
-   * Next code just does sprintf( {}, "%d", n);
-   * It is more elegant to do the next part recursively, but a procedure
-   * call for each digit emitted is considered too costly.
-   */
+#ifdef LOCAL_LABEL_PREFIX
+  *p++ = LOCAL_LABEL_PREFIX;
+#endif
+  *p++ = 'L';
+
+  /* Next code just does sprintf( {}, "%d", n);  */
+  /* Label number.  */
   q = symbol_name_temporary;
-  for (*q++=0; n; q++)		/* emits NOTHING if n starts as 0 */
+  for (*q++ = 0, i = n; i; ++q)
     {
-      know(n>0);		/* We expect n > 0 always */
-      *q = n % 10 + '0';
-      n /= 10;
+      *q = i % 10 + '0';
+      i /= 10;
     }
-  while (( * p ++ = * -- q ))
+  while ((*p = *--q) != '\0')
+    ++p;
+
+  *p++ = LOCAL_LABEL_CHAR;		/* ^B  */
+
+  /* Instance number.  */
+  q = symbol_name_temporary;
+  for (*q++ = 0, i = fb_label_instance (n) + augend; i; ++q)
     {
+      *q = i % 10 + '0';
+      i /= 10;
     }
-  /* The label, as a '\0' ended string, starts at symbol_name_build. */
+  while ((*p++ = *--q) != '\0');
+
+  /* The label, as a '\0' ended string, starts at symbol_name_build.  */
   return (symbol_name_build);
 }
 
+/* Decode name that may have been generated by foo_label_name() above.
+   If the name wasn't generated by foo_label_name(), then return it
+   unaltered.  This is used for error messages.  */
+
+char *
+decode_local_label_name (char *s)
+{
+  char *p;
+  char *symbol_decode;
+  int label_number;
+  int instance_number;
+  char *type;
+  int index = 0;
+
+#ifdef LOCAL_LABEL_PREFIX
+  if (s[index] == LOCAL_LABEL_PREFIX)
+    ++index;
+#endif
+
+  if (s[index] != 'L')
+    return s;
+
+  for (label_number = 0, p = s + index + 1; isdigit (*p); ++p)
+    label_number = (10 * label_number) + *p - '0';
+
+  if (*p == LOCAL_LABEL_CHAR)
+    type = "fb";
+  else
+    return s;
+
+  for (instance_number = 0, p++; isdigit (*p); ++p)
+    instance_number = (10 * instance_number) + *p - '0';
+
+#define MESSAGE_FORMAT "\"%d\" (instance number %d of a %s label)"
+  symbol_decode = obstack_alloc (&notes, strlen (MESSAGE_FORMAT) + 30);
+  sprintf (symbol_decode, MESSAGE_FORMAT, label_number, instance_number, type);
+
+  return symbol_decode;
+}
+
 void
 local_colon(
 int n)	/* just saw "n:" */
 {
-  local_label_counter [n] ++;
-  colon (local_label_name (n, 0));
+  fb_label_instance_inc (n);
+  colon (fb_label_name (n, 0), 1);
 }
 
 /*
@@ -166,11 +335,11 @@ struct frag    *frag)	/* For sy_frag. */
 {
   register symbolS *		symbolP;
   register char *		preserved_copy_of_name;
-  register unsigned int		name_length;
+  register size_t		name_length;
            char *		p;
 
   name_length = strlen(name) + 1;
-  obstack_grow(&notes,name,name_length);
+  obstack_grow(&notes,name,(int)name_length);
   p=obstack_finish(&notes);
   /* obstack_1done( &notes, name, name_length, &p ); */
   preserved_copy_of_name = p;
@@ -184,6 +353,7 @@ struct frag    *frag)	/* For sy_frag. */
   symbolP -> sy_value		= value;
   symbolP -> sy_frag		= frag;
   symbolP -> sy_prev_by_index	= NULL; /* Don't know what this is yet. */
+  symbolP -> sy_has_been_resolved = 0;
   symbolP -> sy_next		= NULL;	/* End of chain. */
   symbolP -> sy_forward		= NULL; /* JF */
   symbolP -> expression		= NULL;
@@ -206,6 +376,17 @@ struct frag    *frag)	/* For sy_frag. */
 
   return (symbolP);
 }
+
+/* FROM line 136 */
+symbolS *
+symbol_create (const char *name, /* It is copied, the caller can destroy/modify.  */
+	       segT segment,	/* Segment identifier (SEG_<something>).  */
+	       valueT valu,	/* Symbol value.  */
+	       fragS *frag	/* Associated fragment.  */)
+{
+  /* FIXME */
+  return symbol_new ((char *)name, 0, segment, 0, valu, frag);
+}
 
 /*
  *			symbol_assign_index()
@@ -241,8 +422,9 @@ static volatile unsigned int temp;
 
 void
 colon(		/* just seen "x:" - rattle symbols & frags */
-char *sym_name) /* symbol name, as a cannonical string */
+char *sym_name, /* symbol name, as a cannonical string */
 		/* We copy this string: OK to alter later. */
+int local_colon)/* non-zero if called from local_colon() */
 {
   register struct symbol * symbolP; /* symbol we are working with */
 
@@ -252,6 +434,14 @@ char *sym_name) /* symbol name, as a cannonical string */
       as_fatal("with -n a section directive must be seen before assembly "
 	       "can begin");
     }
+  if (inlineasm_checks && local_colon == 0)
+   {
+     if (inlineasm_file_name)
+       as_warn_where_with_column(inlineasm_file_name, inlineasm_line_number,
+		    inlineasm_column_number, "label definition in inlineasm");
+     else
+       as_bad("label definition in inlineasm");
+   }
   if ((symbolP = symbol_table_lookup( sym_name )))
     {
       /*
@@ -265,26 +455,25 @@ char *sym_name) /* symbol name, as a cannonical string */
 	     && ((symbolP->sy_desc) & (~REFERENCE_TYPE)) == 0
 	     */
 	     && (temp & (~(REFERENCE_TYPE | N_WEAK_REF | N_WEAK_DEF |
+			   N_ARM_THUMB_DEF | N_SYMBOL_RESOLVER |
 			   N_NO_DEAD_STRIP | REFERENCED_DYNAMICALLY))) == 0
 	     && symbolP -> sy_value == 0)
 	    {
 	      symbolP -> sy_frag  = frag_now;
-	      symbolP -> sy_value = obstack_next_free(& frags) - frag_now -> fr_literal;
+	      symbolP -> sy_value = (uint32_t)(obstack_next_free(& frags) -
+					       frag_now -> fr_literal);
 	      know( N_UNDF == 0 );
 	      symbolP -> sy_type |= N_SECT; /* keep N_EXT bit */
 	      symbolP -> sy_other = frchain_now->frch_nsect;
 	      symbolP -> sy_desc &= ~REFERENCE_TYPE;
-	      symbolP -> sy_desc &= ~N_WEAK_REF;
+	      symbolP -> sy_desc &= ~(N_WEAK_REF & N_WEAK_DEF);
 	      symbol_assign_index(symbolP);
-	      if((symbolP->sy_desc & N_WEAK_DEF) == N_WEAK_DEF &&
-		 (frchain_now->frch_section.flags & S_COALESCED) != S_COALESCED)
-		  as_fatal("symbol: %s can't be a weak_definition (currently "
-			   "only supported in section of type coalesced)",
-			   sym_name);
 #ifdef NeXT_MOD	/* generate stabs for debugging assembly code */
 	      if(flagseen['g'])
 		  make_stab_for_symbol(symbolP);
 #endif
+	      if(debug_type == DEBUG_DWARF2)
+		  make_subprogram_for_symbol(symbolP);
 	    }
 	  else
 	    {
@@ -315,7 +504,12 @@ char *sym_name) /* symbol name, as a cannonical string */
       if(flagseen['g'])
 	  make_stab_for_symbol(symbolP);
 #endif
+      if(debug_type == DEBUG_DWARF2)
+	  make_subprogram_for_symbol(symbolP);
     }
+#ifdef tc_frob_label
+    tc_frob_label(symbolP);
+#endif
 }
 
 
@@ -410,7 +604,7 @@ make_stab_for_symbol(
 symbolS *symbolP)
 {
     symbolS *stab;
-    int stabnamelen;
+    size_t stabnamelen;
     char *stabname;
 
 	if(symbolP->sy_name[0] == 'L')
@@ -439,6 +633,52 @@ symbolS *symbolP)
 }
 #endif /* NeXT generate stabs for debugging assembly code */
 
+/*
+ * make_subprogram_for_symbol() gathers the info that is needed for each
+ * symbol that will have a dwarf2_subprogram when generating dwarf debugging
+ * info for assembly files.
+ */
+static
+void
+make_subprogram_for_symbol(
+symbolS *symbolP)
+{
+    struct dwarf2_subprogram_info *i;
+    static struct dwarf2_subprogram_info *last_dwarf2_subprogram_info = NULL;
+
+	if(symbolP->sy_name[0] == 'L')
+	    return;
+	if((symbolP->sy_type & N_TYPE) != N_SECT)
+	    return;
+	if(symbolP->sy_other != text_nsect)
+	    return;
+
+	i = xmalloc(sizeof(struct dwarf2_subprogram_info));
+	i->name = symbolP->sy_name;
+	if(i->name[0] == '_')
+	    i->name++;
+	i->file_number = dwarf2_file_number;
+	i->line_number = logical_input_line;
+	/*
+	 * We can't used the symbolP directly as it may have the N_ARM_THUMB_DEF
+	 * bit set.  And that will cause the AT_high_pc and AT_low_pc values to
+	 * have the low bit set after relocation producing bad dwarf.  So we
+	 * create a temporary symbol that will not have the N_ARM_THUMB_DEF bit
+	 * set.
+	 */
+	i->symbol = symbol_temp_new(symbolP->sy_other, symbolP->sy_value,
+				    symbolP->sy_frag);
+	i->next = NULL;
+	if(dwarf2_subprograms_info == NULL){
+	    dwarf2_subprograms_info = i;
+	    last_dwarf2_subprogram_info = i;
+	}
+	else{
+	    last_dwarf2_subprogram_info->next = i;
+	    last_dwarf2_subprogram_info = i;
+	}
+}
+
 /*
  * indirect_symbol_new()
  *
@@ -451,16 +691,16 @@ isymbolS *
 indirect_symbol_new(
 char	       *name,	  /* We copy this: OK to alter your copy. */
 struct frag    *frag,	  /* For sy_frag. */
-unsigned long	offset)	  /* Offset from frag address. */
+uint32_t	offset)	  /* Offset from frag address. */
 {
     isymbolS *isymbolP;
     char *preserved_copy_of_name;
-    unsigned long name_length;
+    size_t name_length;
     char *p;
     struct frag *fr_next;
     symbolS *symbolP;
 #ifdef CHECK_INDIRECTS
-    unsigned long stride, fr_fix;
+    uint32_t stride, fr_fix;
 #endif
 
 	/*
@@ -483,7 +723,7 @@ unsigned long	offset)	  /* Offset from frag address. */
 	}
 
 	name_length = strlen(name) + 1;
-	obstack_grow(&notes, name, name_length);
+	obstack_grow(&notes, name, (int)name_length);
 	p = obstack_finish(&notes);
 	preserved_copy_of_name = p;
 	p = obstack_alloc(&notes, sizeof(struct indirect_symbol));
@@ -514,7 +754,7 @@ unsigned long	offset)	  /* Offset from frag address. */
 	       S_SYMBOL_STUBS)
 		stride = frchain_now->frch_section.reserved2;
 	    else
-		stride = sizeof(unsigned long);
+		stride = sizeof(uint32_t);
 	    if(frag == frchain_now->frch_isym_last->isy_frag){
 		if(offset - frchain_now->frch_isym_last->isy_offset != stride)
 		    as_bad("missing or bad indirect symbol for section "
@@ -568,4 +808,99 @@ unsigned long	offset)	  /* Offset from frag address. */
 	return(isymbolP);
 }
 
+const char *
+S_GET_NAME (symbolS *s)
+{
+  return s->sy_name;
+}
+
+int
+S_IS_DEFINED (symbolS *s)
+{
+  return (s->sy_type & N_TYPE) != N_UNDF;
+}
+
+/* FROM line 2317 */
+#ifdef TC_SYMFIELD_TYPE
+
+/* Get a pointer to the processor information for a symbol.  */
+
+TC_SYMFIELD_TYPE *
+symbol_get_tc (symbolS *s)
+{
+  return &s->sy_tc;
+}
+
+/* Set the processor information for a symbol.  */
+
+void
+symbol_set_tc (symbolS *s, TC_SYMFIELD_TYPE *o)
+{
+  s->sy_tc = *o;
+}
+
+#endif /* TC_SYMFIELD_TYPE */
+
+int
+S_IS_LOCAL (symbolS *s)
+{
+    const char *name;
+
+	name = S_GET_NAME (s);
+	if(name == NULL)
+	    return(1);
+
+	if(name[0] == 'L' && flagseen['L'] == FALSE)
+	    return(1);
+	else
+	    return(0);
+}
+
+fragS *
+symbol_get_frag (symbolS *s)
+{
+	return(s->sy_frag);
+}
+
+/*
+ * symbol_temp_new(), symbol_temp_new_now() are used by dwarf2dbg.c to make
+ * symbols in dwarf sections.  symbol_temp_make() is used to make an undefined
+ * symbol that its values are later set by symbol_set_value_now() to the current
+ * address in a dwarf section.
+ *
+ * These are used in expressions, so the expression values can be put out in
+ * dwarf section contents.
+ */
+symbolS *
+symbol_temp_new(
+segT nsect,
+valueT value,
+struct frag *frag)
+{
+    return(symbol_new(FAKE_LABEL_NAME, N_SECT, nsect, 0, value, frag));
+}
+
+symbolS *
+symbol_temp_new_now(void)
+{
+    return(symbol_temp_new(now_seg, (valueT)frag_now_fix(), frag_now));
+}
+
+symbolS *
+symbol_temp_make(void)
+{
+    return(symbol_new(FAKE_LABEL_NAME, N_UNDF, 0, 0, 0, & zero_address_frag));
+}
+
+/* Set the value of SYM to the current position in the current segment.  */
+void
+symbol_set_value_now(
+symbolS *sym)
+{
+    sym->sy_type = N_SECT;
+    sym->sy_other = now_seg;
+    sym->sy_value = (uint32_t)frag_now_fix();
+    sym->sy_frag = frag_now;
+}
+
 /* end: symbols.c */
