@@ -98,6 +98,11 @@ __private_extern__ void swap_fat_arch(
     uint32_t nfat_arch,
     enum byte_sex target_byte_sex);
 
+__private_extern__ void swap_fat_arch_64(
+    struct fat_arch_64 *fat_archs64,
+    uint32_t nfat_arch,
+    enum byte_sex target_byte_sex);
+
 __private_extern__ void swap_mach_header(
     struct mach_header *mh,
     enum byte_sex target_byte_sex);
@@ -312,6 +317,10 @@ __private_extern__ void swap_arm_thread_state64_t(
     arm_thread_state64_t *cpu,
     enum byte_sex target_byte_sex);
 
+__private_extern__ void swap_arm_exception_state64_t(
+    arm_exception_state64_t *except,
+    enum byte_sex target_byte_sex);
+
 __private_extern__ void swap_ident_command(
     struct ident_command *id_cmd,
     enum byte_sex target_byte_sex);
@@ -372,6 +381,19 @@ __private_extern__ void swap_source_version_command(
     struct source_version_command *sv,
     enum byte_sex target_byte_sex);
 
+__private_extern__ void swap_note_command(
+    struct note_command *nc,
+    enum byte_sex target_byte_sex);
+
+__private_extern__ void swap_build_version_command(
+    struct build_version_command *bv,
+    enum byte_sex target_byte_sex);
+
+__private_extern__ void swap_build_tool_version(
+    struct build_tool_version *bt,
+    uint32_t ntools,
+    enum byte_sex target_byte_sex);
+
 __private_extern__ void swap_nlist(
     struct nlist *symbols,
     uint32_t nsymbols,
@@ -387,6 +409,11 @@ __private_extern__ void swap_ranlib(
     uint32_t nranlibs,
     enum byte_sex target_byte_sex);
 
+__private_extern__ void swap_ranlib_64(
+    struct ranlib_64 *ranlibs,
+    uint64_t nranlibs,
+    enum byte_sex target_byte_sex);
+
 __private_extern__ void swap_relocation_info(
     struct relocation_info *relocs,
     uint32_t nrelocs,
@@ -449,6 +476,6 @@ __private_extern__ enum bool swap_object_headers(
  */
 __private_extern__ enum byte_sex get_toc_byte_sex(
     char *addr,
-    uint32_t size);
+    uint64_t size);
 
 #endif /* _STUFF_BYTESEX_H_ */
