@@ -21,6 +21,6 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 __private_extern__ enum bool get_version_number(
-    char *flag,
-    char *argument,
+    const char *flag,
+    const char *argument,
     uint32_t *value);
