@@ -40,6 +40,7 @@
 #include <mach/i386/thread_status.h>
 #include <mach/hppa/thread_status.h>
 #include <mach/sparc/thread_status.h>
+#include <mach/arm/thread_status.h>
 #include "stuff/bool.h"
 #include "stuff/bytesex.h"
 #include "stuff/errors.h"
@@ -74,6 +75,8 @@ struct load_command *load_commands)
     struct fvmlib_command *fl;
     struct thread_command *ut;
     struct ident_command *id;
+    struct entry_point_command *ep;
+    struct source_version_command *sv;
     struct dylib_command *dl;
     struct sub_framework_command *sub;
     struct sub_umbrella_command *usub;
@@ -86,8 +89,19 @@ struct load_command *load_commands)
     struct twolevel_hints_command *hints;
     struct prebind_cksum_command *cs;
     struct uuid_command *uuid;
-    unsigned long flavor, count, nflavor;
-    char *p, *state;
+    struct linkedit_data_command *ld;
+    struct rpath_command *rpath;
+    struct encryption_info_command *ec;
+    struct encryption_info_command_64 *ec64;
+    struct linker_option_command *lo;
+    struct dyld_info_command *dc;
+    struct version_min_command *vc;
+    struct build_version_command *bv;
+    struct build_tool_version *btv;
+    struct note_command *nc;
+    uint32_t flavor, count;
+    unsigned long nflavor;
+    char *p, *state, *cmd_name;
 
 	magic = *((uint32_t *)mach_header);
 	if(magic == MH_MAGIC){
@@ -205,24 +219,36 @@ struct load_command *load_commands)
 		break;
 
 	    case LC_ID_DYLIB:
+		cmd_name = "LC_ID_DYLIB";
+		goto check_dylib_command;
 	    case LC_LOAD_DYLIB:
+		cmd_name = "LC_LOAD_DYLIB";
+		goto check_dylib_command;
 	    case LC_LOAD_WEAK_DYLIB:
+		cmd_name = "LC_LOAD_WEAK_DYLIB";
+		goto check_dylib_command;
+	    case LC_REEXPORT_DYLIB:
+		cmd_name = "LC_REEXPORT_DYLIB";
+		goto check_dylib_command;
+	    case LC_LOAD_UPWARD_DYLIB:
+		cmd_name = "LC_LOAD_UPWARD_DYLIB";
+		goto check_dylib_command;
+	    case LC_LAZY_LOAD_DYLIB:
+		cmd_name = "LC_LAZY_LOAD_DYLIB";
+		goto check_dylib_command;
+check_dylib_command:
 		dl = (struct dylib_command *)lc;
 		if(dl->cmdsize < sizeof(struct dylib_command)){
 		    error("in swap_object_headers(): malformed load commands "
 			  "(%s command %lu has too small cmdsize field)",
-			  dl->cmd == LC_ID_DYLIB ? "LC_ID_DYLIB" :
-			  (dl->cmd == LC_LOAD_DYLIB ? "LC_LOAD_DYLIB" :
-			  "LC_LOAD_WEAK_DYLIB"), i);
+			  cmd_name, i);
 		    return(FALSE);
 		}
 		if(dl->dylib.name.offset >= dl->cmdsize){
 		    error("in swap_object_headers(): truncated or malformed "
 			  "load commands (name.offset field of %s command %lu "
 			  "extends past the end of all load commands)",
-			  dl->cmd == LC_ID_DYLIB ? "LC_ID_DYLIB" :
-			  (dl->cmd == LC_LOAD_DYLIB ? "LC_LOAD_DYLIB" :
-			  "LC_LOAD_WEAK_DYLIB"), i);
+			  cmd_name, i);
 		    return(FALSE);
 		}
 		break;
@@ -320,21 +346,27 @@ struct load_command *load_commands)
 		break;
 
 	    case LC_ID_DYLINKER:
+		cmd_name = "LC_ID_DYLINKER";
+		goto check_dylinker_command;
 	    case LC_LOAD_DYLINKER:
+		cmd_name = "LC_LOAD_DYLINKER";
+		goto check_dylinker_command;
+	    case LC_DYLD_ENVIRONMENT:
+		cmd_name = "LC_DYLD_ENVIRONMENT";
+		goto check_dylinker_command;
+check_dylinker_command:
 		dyld = (struct dylinker_command *)lc;
 		if(dyld->cmdsize < sizeof(struct dylinker_command)){
 		    error("in swap_object_headers(): malformed load commands "
 			  "(%s command %lu has too small cmdsize field)",
-			  dyld->cmd == LC_ID_DYLINKER ? "LC_ID_DYLINKER" :
-			  "LC_LOAD_DYLINKER", i);
+			  cmd_name, i);
 		    return(FALSE);
 		}
 		if(dyld->name.offset >= dyld->cmdsize){
 		    error("in swap_object_headers(): truncated or malformed "
 			  "load commands (name.offset field of %s command %lu "
 			  "extends past the end of all load commands)",
-			  dyld->cmd == LC_ID_DYLINKER ? "LC_ID_DYLINKER" :
-			  "LC_LOAD_DYLINKER", i);
+			  cmd_name, i);
 		    return(FALSE);
 		}
 		break;
@@ -352,10 +384,10 @@ struct load_command *load_commands)
 		    nflavor = 0;
 		    p = (char *)ut + ut->cmdsize;
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case M68K_THREAD_STATE_REGS:
 			    if(count != M68K_THREAD_STATE_REGS_COUNT){
@@ -403,7 +435,7 @@ struct load_command *load_commands)
 			default:
 			    error("in swap_object_headers(): malformed "
 				"load commands (unknown "
-				"flavor %lu for flavor number %lu in %s command"
+				"flavor %u for flavor number %lu in %s command"
 				" %lu can't byte swap it)", flavor, nflavor,
 				ut->cmd == LC_UNIXTHREAD ? "LC_UNIXTHREAD" :
 				"LC_THREAD", i);
@@ -424,10 +456,10 @@ struct load_command *load_commands)
 		    nflavor = 0;
 		    p = (char *)ut + ut->cmdsize;
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case PPC_THREAD_STATE:
 			    if(count != PPC_THREAD_STATE_COUNT){
@@ -488,7 +520,7 @@ struct load_command *load_commands)
 			default:
 			    error("in swap_object_headers(): malformed "
 				"load commands (unknown "
-				"flavor %lu for flavor number %lu in %s command"
+				"flavor %u for flavor number %lu in %s command"
 				" %lu can't byte swap it)", flavor, nflavor,
 				ut->cmd == LC_UNIXTHREAD ? "LC_UNIXTHREAD" :
 				"LC_THREAD", i);
@@ -507,10 +539,10 @@ struct load_command *load_commands)
 		    nflavor = 0;
 		    p = (char *)ut + ut->cmdsize;
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case M88K_THREAD_STATE_GRF:
 			    if(count != M88K_THREAD_STATE_GRF_COUNT){
@@ -571,7 +603,7 @@ struct load_command *load_commands)
 			default:
 			    error("in swap_object_headers(): malformed "
 				"load commands (unknown "
-				"flavor %lu for flavor number %lu in %s command"
+				"flavor %u for flavor number %lu in %s command"
 				" %lu can't byte swap it)", flavor, nflavor,
 				ut->cmd == LC_UNIXTHREAD ? "LC_UNIXTHREAD" :
 				"LC_THREAD", i);
@@ -587,10 +619,10 @@ struct load_command *load_commands)
 		    nflavor = 0;
 		    p = (char *)ut + ut->cmdsize;
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case I860_THREAD_STATE_REGS:
 			    if(count != I860_THREAD_STATE_REGS_COUNT){
@@ -609,7 +641,7 @@ struct load_command *load_commands)
 			default:
 			    error("in swap_object_headers(): malformed "
 				"load commands (unknown "
-				"flavor %lu for flavor number %lu in %s command"
+				"flavor %u for flavor number %lu in %s command"
 				" %lu can't byte swap it)", flavor, nflavor,
 				ut->cmd == LC_UNIXTHREAD ? "LC_UNIXTHREAD" :
 				"LC_THREAD", i);
@@ -644,11 +676,11 @@ struct load_command *load_commands)
 		    nflavor = 0;
 		    p = (char *)ut + ut->cmdsize;
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			switch(flavor){
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			switch((int)flavor){
 			case i386_THREAD_STATE:
 /* current i386 thread states */
 #if i386_THREAD_STATE == 1
@@ -771,7 +803,7 @@ struct load_command *load_commands)
 			default:
 			    error("in swap_object_headers(): malformed "
 				"load commands (unknown "
-				"flavor %lu for flavor number %lu in %s command"
+				"flavor %u for flavor number %lu in %s command"
 				" %lu can't byte swap it)", flavor, nflavor,
 				ut->cmd == LC_UNIXTHREAD ? "LC_UNIXTHREAD" :
 				"LC_THREAD", i);
@@ -789,10 +821,10 @@ struct load_command *load_commands)
 		    nflavor = 0;
 		    p = (char *)ut + ut->cmdsize;
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case HPPA_INTEGER_THREAD_STATE:
 			    if(count != HPPA_INTEGER_THREAD_STATE_COUNT){
@@ -839,7 +871,7 @@ struct load_command *load_commands)
 			default:
 			    error("in swap_object_headers(): malformed "
 				"load commands (unknown "
-				"flavor %lu for flavor number %lu in %s command"
+				"flavor %u for flavor number %lu in %s command"
 				" %lu can't byte swap it)", flavor, nflavor,
 				ut->cmd == LC_UNIXTHREAD ? "LC_UNIXTHREAD" :
 				"LC_THREAD", i);
@@ -856,10 +888,10 @@ struct load_command *load_commands)
 		  nflavor = 0;
 		  p = (char *)ut + ut->cmdsize;
 		  while (state < p) {
-		    flavor = *((unsigned long *) state);
-		    state += sizeof(unsigned long);
-		    count = *((unsigned int *) state);
-		    state += sizeof(unsigned long);
+		    flavor = *((uint32_t *) state);
+		    state += sizeof(uint32_t);
+		    count = *((uint32_t *) state);
+		    state += sizeof(uint32_t);
 		    switch (flavor) {
 		    case SPARC_THREAD_STATE_REGS:
 		      if (count != SPARC_THREAD_STATE_REGS_COUNT) {
@@ -893,13 +925,122 @@ struct load_command *load_commands)
 		  }
 		  break;
 		}
-		    
+	    	if(cputype == CPU_TYPE_ARM){
+		    arm_thread_state_t *cpu;
+
+		    nflavor = 0;
+		    p = (char *)ut + ut->cmdsize;
+		    while(state < p){
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			switch(flavor){
+			case ARM_THREAD_STATE:
+			    if(count != ARM_THREAD_STATE_COUNT){
+				error("in swap_object_headers(): malformed "
+				    "load commands (count "
+				    "not ARM_THREAD_STATE_COUNT for "
+				    "flavor number %lu which is a ARM_THREAD_"
+				    "STATE flavor in %s command %lu)",
+				    nflavor, ut->cmd == LC_UNIXTHREAD ? 
+				    "LC_UNIXTHREAD" : "LC_THREAD", i);
+				return(FALSE);
+			    }
+			    cpu = (arm_thread_state_t *)state;
+			    state += sizeof(arm_thread_state_t);
+			    break;
+			default:
+			    error("in swap_object_headers(): malformed load "
+				  "commands (unknown flavor for flavor number "
+				  "%lu in %s command %lu can't byte swap it)",
+				  nflavor, ut->cmd == LC_UNIXTHREAD ?
+				  "LC_UNIXTHREAD" : "LC_THREAD", i);
+			    return(FALSE);
+			}
+			nflavor++;
+		    }
+		    break;
+		}
+	    	if(cputype == CPU_TYPE_ARM64){
+		    arm_thread_state64_t *cpu;
+
+		    nflavor = 0;
+		    p = (char *)ut + ut->cmdsize;
+		    while(state < p){
+			flavor = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			state += sizeof(uint32_t);
+			switch(flavor){
+			case ARM_THREAD_STATE:
+			    if(count != ARM_THREAD_STATE_COUNT){
+				error("in swap_object_headers(): malformed "
+				    "load commands (count "
+				    "not ARM_THREAD_STATE_COUNT for "
+				    "flavor number %lu which is a ARM_THREAD_"
+				    "STATE flavor in %s command %lu)",
+				    nflavor, ut->cmd == LC_UNIXTHREAD ? 
+				    "LC_UNIXTHREAD" : "LC_THREAD", i);
+				return(FALSE);
+			    }
+			    cpu = (arm_thread_state64_t *)state;
+			    state += sizeof(arm_thread_state64_t);
+			    break;
+			default:
+			    error("in swap_object_headers(): malformed load "
+				  "commands (unknown flavor for flavor number "
+				  "%lu in %s command %lu can't byte swap it)",
+				  nflavor, ut->cmd == LC_UNIXTHREAD ?
+				  "LC_UNIXTHREAD" : "LC_THREAD", i);
+			    return(FALSE);
+			}
+			nflavor++;
+		    }
+		    break;
+		}
 		error("in swap_object_headers(): malformed load commands "
 		    "(unknown cputype (%d) and cpusubtype (%d) of object and "
                     "can't byte swap %s command %lu)", cputype, 
 		    cpusubtype, ut->cmd == LC_UNIXTHREAD ?
 		    "LC_UNIXTHREAD" : "LC_THREAD", i);
 		return(FALSE);
+
+	    case LC_MAIN:
+		ep = (struct entry_point_command *)lc;
+		if((char *)ep + ep->cmdsize >
+		   (char *)load_commands + sizeofcmds){
+		    error("in swap_object_headers(): truncated or malformed "
+			"load commands (cmdsize field of LC_MAIN command %lu "
+			"extends past the end of the load commands)", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_SOURCE_VERSION:
+		sv = (struct source_version_command *)lc;
+		if((char *)sv + sv->cmdsize >
+		   (char *)load_commands + sizeofcmds){
+		    error("in swap_object_headers(): truncated or malformed "
+			"load commands (cmdsize field of LC_SOURCE_VERSION "
+			"command %lu extends past the end of the load "
+			"commands)", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_NOTE:
+		nc = (struct note_command *)lc;
+		if((char *)nc + nc->cmdsize >
+		   (char *)load_commands + sizeofcmds){
+		    error("in swap_object_headers(): truncated or malformed "
+			"load commands (cmdsize field of LC_NOTE "
+			"command %lu extends past the end of the load "
+			"commands)", i);
+		    return(FALSE);
+		}
+		break;
+
 	    case LC_IDENT:
 		id = (struct ident_command *)lc;
 		if((char *)id + id->cmdsize >
@@ -960,6 +1101,174 @@ struct load_command *load_commands)
 		}
 		break;
 
+	    case LC_CODE_SIGNATURE:
+		ld = (struct linkedit_data_command *)lc;
+		if(ld->cmdsize != sizeof(struct linkedit_data_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_CODE_SIGNATURE command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_SEGMENT_SPLIT_INFO:
+		ld = (struct linkedit_data_command *)lc;
+		if(ld->cmdsize != sizeof(struct linkedit_data_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_SEGMENT_SPLIT_INFO command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_FUNCTION_STARTS:
+		ld = (struct linkedit_data_command *)lc;
+		if(ld->cmdsize != sizeof(struct linkedit_data_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_FUNCTION_STARTS command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_DATA_IN_CODE:
+		ld = (struct linkedit_data_command *)lc;
+		if(ld->cmdsize != sizeof(struct linkedit_data_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_DATA_IN_CODE command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_DYLIB_CODE_SIGN_DRS:
+		ld = (struct linkedit_data_command *)lc;
+		if(ld->cmdsize != sizeof(struct linkedit_data_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_DYLIB_CODE_SIGN_DRS command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_LINKER_OPTIMIZATION_HINT:
+		ld = (struct linkedit_data_command *)lc;
+		if(ld->cmdsize != sizeof(struct linkedit_data_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_LINKER_OPTIMIZATION_HINT command %lu has "
+			  "incorrect cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_VERSION_MIN_MACOSX:
+		vc = (struct version_min_command *)lc;
+		if(vc->cmdsize != sizeof(struct version_min_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_VERSION_MIN_MACOSX command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_VERSION_MIN_IPHONEOS:
+		vc = (struct version_min_command *)lc;
+		if(vc->cmdsize != sizeof(struct version_min_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_VERSION_MIN_IPHONEOS command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_VERSION_MIN_TVOS:
+		vc = (struct version_min_command *)lc;
+		if(vc->cmdsize != sizeof(struct version_min_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_VERSION_MIN_ command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_VERSION_MIN_WATCHOS:
+		vc = (struct version_min_command *)lc;
+		if(vc->cmdsize != sizeof(struct version_min_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_VERSION_MIN_WATCHOS command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_BUILD_VERSION:
+		bv = (struct build_version_command *)lc;
+		if(bv->cmdsize != sizeof(struct build_version_command) +
+				bv->ntools * sizeof(struct build_tool_version)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_BUILD_VERSION command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_RPATH:
+		rpath = (struct rpath_command *)lc;
+		if(rpath->cmdsize < sizeof(struct rpath_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_RPATH command %lu has too small cmdsize field)",
+			  i);
+		    return(FALSE);
+		}
+		if(rpath->path.offset >= rpath->cmdsize){
+		    error("in swap_object_headers(): truncated or malformed "
+			  "load commands (path.offset field of LC_RPATH "
+			  "command %lu extends past the end of all load "
+			  "commands)", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_ENCRYPTION_INFO:
+		ec = (struct encryption_info_command *)lc;
+		if(ec->cmdsize != sizeof(struct encryption_info_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_ENCRYPTION_INFO command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_ENCRYPTION_INFO_64:
+		ec64 = (struct encryption_info_command_64 *)lc;
+		if(ec64->cmdsize != sizeof(struct encryption_info_command_64)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_ENCRYPTION_INFO_64 command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_LINKER_OPTION:
+		lo = (struct linker_option_command *)lc;
+		if(lo->cmdsize < sizeof(struct linker_option_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_LINKER_OPTION command %lu is too small", i);
+		    return(FALSE);
+		}
+		break;
+
+	    case LC_DYLD_INFO:
+	    case LC_DYLD_INFO_ONLY:
+		dc = (struct dyld_info_command *)lc;
+		if(dc->cmdsize != sizeof(struct dyld_info_command)){
+		    error("in swap_object_headers(): malformed load commands "
+			  "(LC_DYLD_INFO command %lu has incorrect "
+			  "cmdsize", i);
+		    return(FALSE);
+		}
+		break;
+
 	    default:
 		error("in swap_object_headers(): malformed load commands "
 		      "(unknown load command %lu)", i);
@@ -1031,6 +1340,9 @@ struct load_command *load_commands)
 	    case LC_ID_DYLIB:
 	    case LC_LOAD_DYLIB:
 	    case LC_LOAD_WEAK_DYLIB:
+	    case LC_REEXPORT_DYLIB:
+	    case LC_LOAD_UPWARD_DYLIB:
+	    case LC_LAZY_LOAD_DYLIB:
 		dl = (struct dylib_command *)lc;
 		swap_dylib_command(dl, target_byte_sex);
 		break;
@@ -1062,6 +1374,7 @@ struct load_command *load_commands)
 
 	    case LC_ID_DYLINKER:
 	    case LC_LOAD_DYLINKER:
+	    case LC_DYLD_ENVIRONMENT:
 		dyld = (struct dylinker_command *)lc;
 		swap_dylinker_command(dyld, target_byte_sex);
 		break;
@@ -1079,12 +1392,12 @@ struct load_command *load_commands)
 		    struct m68k_thread_state_user_reg *user_reg;
 
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(flavor);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(count);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case M68K_THREAD_STATE_REGS:
 			    cpu = (struct m68k_thread_state_regs *)state;
@@ -1116,12 +1429,12 @@ struct load_command *load_commands)
 		    ppc_exception_state_t *except;
 
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(flavor);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(count);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case PPC_THREAD_STATE:
 			    cpu = (ppc_thread_state_t *)state;
@@ -1153,12 +1466,12 @@ struct load_command *load_commands)
 		    m88110_thread_state_impl_t *spu;
 
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(flavor);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(count);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case M88K_THREAD_STATE_GRF:
 			    cpu = (m88k_thread_state_grf_t *)state;
@@ -1192,12 +1505,12 @@ struct load_command *load_commands)
 		    struct i860_thread_state_regs *cpu;
 
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(flavor);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(count);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case I860_THREAD_STATE_REGS:
 			    cpu = (struct i860_thread_state_regs *)state;
@@ -1231,13 +1544,13 @@ struct load_command *load_commands)
 #endif /* i386_THREAD_STATE == -1 */
 
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(flavor);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(count);
-			state += sizeof(unsigned long);
-			switch(flavor){
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
+			switch((int)flavor){
 			case i386_THREAD_STATE:
 /* current i386 thread states */
 #if i386_THREAD_STATE == 1
@@ -1300,12 +1613,12 @@ struct load_command *load_commands)
 		    struct hp_pa_fp_thread_state *fpu;
 
 		    while(state < p){
-			flavor = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(flavor);
-			state += sizeof(unsigned long);
-			count = *((unsigned long *)state);
-			*((unsigned long *)state) = SWAP_LONG(count);
-			state += sizeof(unsigned long);
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
 			switch(flavor){
 			case HPPA_INTEGER_THREAD_STATE:
 			    cpu = (struct hp_pa_integer_thread_state *)state;
@@ -1335,12 +1648,12 @@ struct load_command *load_commands)
 		  struct sparc_thread_state_fpu *fpu;
 
 		  while (state < p) {
-		    flavor = *((unsigned long *) state);
-		    *((unsigned long *) state) = SWAP_LONG(flavor);
-		    state += sizeof(unsigned long);
+		    flavor = *((uint32_t *) state);
+		    *((uint32_t *) state) = SWAP_INT(flavor);
+		    state += sizeof(uint32_t);
 		    count = *((unsigned int *) state);
-		    *((unsigned int *) state) = SWAP_LONG(count);
-		    state += sizeof(unsigned long);
+		    *((unsigned int *) state) = SWAP_INT(count);
+		    state += sizeof(uint32_t);
 		    switch (flavor) {
 		    case SPARC_THREAD_STATE_REGS:
 		      cpu = (struct sparc_thread_state_regs *) state;
@@ -1356,6 +1669,61 @@ struct load_command *load_commands)
 		  }
 		  break;
 		}
+	    	if(cputype == CPU_TYPE_ARM){
+		    arm_thread_state_t *cpu;
+
+		    while(state < p){
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
+			switch(flavor){
+			case ARM_THREAD_STATE:
+			    cpu = (arm_thread_state_t *)state;
+			    swap_arm_thread_state_t(cpu, target_byte_sex);
+			    state += sizeof(arm_thread_state_t);
+			    break;
+			}
+		    }
+		    break;
+		}
+	    	if(cputype == CPU_TYPE_ARM64){
+		    arm_thread_state64_t *cpu;
+
+		    while(state < p){
+			flavor = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(flavor);
+			state += sizeof(uint32_t);
+			count = *((uint32_t *)state);
+			*((uint32_t *)state) = SWAP_INT(count);
+			state += sizeof(uint32_t);
+			switch(flavor){
+			case ARM_THREAD_STATE64:
+			    cpu = (arm_thread_state64_t *)state;
+			    swap_arm_thread_state64_t(cpu, target_byte_sex);
+			    state += sizeof(arm_thread_state64_t);
+			    break;
+			}
+		    }
+		    break;
+		}
+		break;
+
+	    case LC_MAIN:
+		ep = (struct entry_point_command *)lc;
+		swap_entry_point_command(ep, target_byte_sex);
+		break;
+
+	    case LC_SOURCE_VERSION:
+		sv = (struct source_version_command *)lc;
+		swap_source_version_command(sv, target_byte_sex);
+		break;
+
+	    case LC_NOTE:
+		nc = (struct note_command *)lc;
+		swap_note_command(nc, target_byte_sex);
 		break;
 
 	    case LC_IDENT:
@@ -1387,6 +1755,59 @@ struct load_command *load_commands)
 		uuid = (struct uuid_command *)lc;
 		swap_uuid_command(uuid, target_byte_sex);
 		break;
+
+	    case LC_CODE_SIGNATURE:
+	    case LC_SEGMENT_SPLIT_INFO:
+	    case LC_FUNCTION_STARTS:
+	    case LC_DATA_IN_CODE:
+	    case LC_DYLIB_CODE_SIGN_DRS:
+	    case LC_LINKER_OPTIMIZATION_HINT:
+	    case LC_DYLD_EXPORTS_TRIE:
+	    case LC_DYLD_CHAINED_FIXUPS:
+		ld = (struct linkedit_data_command *)lc;
+		swap_linkedit_data_command(ld, target_byte_sex);
+		break;
+
+	    case LC_RPATH:
+		rpath = (struct rpath_command *)lc;
+		swap_rpath_command(rpath, target_byte_sex);
+		break;
+
+	    case LC_ENCRYPTION_INFO:
+		ec = (struct encryption_info_command *)lc;
+		swap_encryption_command(ec, target_byte_sex);
+		break;
+		
+	    case LC_ENCRYPTION_INFO_64:
+		ec64 = (struct encryption_info_command_64 *)lc;
+		swap_encryption_command_64(ec64, target_byte_sex);
+		break;
+		
+	    case LC_LINKER_OPTION:
+		lo = (struct linker_option_command *)lc;
+		swap_linker_option_command(lo, target_byte_sex);
+		break;
+		
+	    case LC_DYLD_INFO:
+	    case LC_DYLD_INFO_ONLY:
+		dc = (struct dyld_info_command *)lc;
+		swap_dyld_info_command(dc, target_byte_sex);
+		break;
+		
+	    case LC_VERSION_MIN_MACOSX:
+	    case LC_VERSION_MIN_IPHONEOS:
+	    case LC_VERSION_MIN_WATCHOS:
+	    case LC_VERSION_MIN_TVOS:
+		vc = (struct version_min_command *)lc;
+		swap_version_min_command(vc, target_byte_sex);
+		break;
+
+	    case LC_BUILD_VERSION:
+		bv = (struct build_version_command *)lc;
+		btv = (struct build_tool_version *)
+		      ((char *)bv + sizeof(struct build_version_command));
+		swap_build_tool_version(btv, bv->ntools, target_byte_sex);
+		swap_build_version_command(bv, target_byte_sex);
 	    }
 
 	    lc = (struct load_command *)((char *)lc + l.cmdsize);
