@@ -33,7 +33,7 @@
 #include "stuff/hash_string.h"
 #include "stuff/allocate.h"
 #include "stuff/errors.h"
-#include "stuff/round.h"
+#include "stuff/rnd.h"
 #include "stuff/reloc.h"
 
 char *progname = NULL;	/* name of the program for error messages (argv[0]) */
@@ -50,7 +50,7 @@ char **list_filenames = NULL;
  */
 struct symbol {
     char *name;			/* name of the symbol */
-    long type;			/* type of this symbol (N_INDR or N_UNDF) */
+    int32_t type;		/* type of this symbol (N_INDR or N_UNDF) */
     char *indr;			/* name for indirection if N_INDR type */
     struct indr_object *io;	/* pointer to indr_object for this */
     struct symbol *next;	/* next symbol in the hash chain */
@@ -69,7 +69,7 @@ struct indr_object {
     char *indr;			/* the name of the indirect symbol */
     char *undef;		/* the name of the undefined symbol */
     enum bool existing_symbol;
-    unsigned long index;
+    uint32_t index;
 };
 
 /*
@@ -82,8 +82,8 @@ struct indr_object {
 #define INITIAL_LIST_SIZE	250
 struct list {
     struct indr_object **list;	/* the order list */
-    unsigned long used;		/* the number used in the list */
-    unsigned long size;		/* the current size of the list */
+    uint32_t used;		/* the number used in the list */
+    uint32_t size;		/* the current size of the list */
 };
 static struct list indr_list;
 
@@ -94,13 +94,13 @@ static struct list indr_list;
 #define INITIAL_STRING_TABLE_SIZE	40960
 static struct {
     char *strings;
-    unsigned long size;
-    unsigned long index;
+    uint32_t size;
+    uint32_t index;
 } string_table;
 
 struct undef_map {
     enum bool old_symbol;
-    unsigned long index;
+    uint32_t index;
     struct nlist symbol;
 };
 static struct undef_map *undef_map;
@@ -116,9 +116,9 @@ static void process_list(
 
 static void translate_input(
     struct arch *archs,
-    unsigned long narchs,
+    uint32_t narchs,
     struct arch_flag *arch_flags,
-    unsigned long narch_flags,
+    uint32_t narch_flags,
     enum bool all_archs,
     enum bool nflag);
 
@@ -144,7 +144,7 @@ static void make_indr_objects(
 
 static void enter_symbol(
     char *name,
-    long type,
+    int32_t type,
     char *indr,
     struct indr_object *io);
 
@@ -160,7 +160,7 @@ static struct indr_object *enter_object(
 static void start_string_table(
     void);
 
-static long add_to_string_table(
+static int32_t add_to_string_table(
     char *p);
 
 static void end_string_table(
@@ -170,6 +170,10 @@ static void add_list(
     struct list *list,
     struct indr_object *item);
 
+/* apple_version is created by the libstuff/Makefile */
+extern char apple_version[];
+char *version = apple_version;
+
 /*
  * The indr(l) program takes the following options:
  *
@@ -191,15 +195,15 @@ char *argv[],
 char *envp[])
 {
     int i;
-    unsigned long j;
+    uint32_t j;
     enum bool no_flags_left;
     char *list_filename, *output_file, *input_file;
     struct arch_flag *arch_flags;
-    unsigned long narch_flags;
+    uint32_t narch_flags;
     enum bool all_archs;
     enum bool nflag;
     struct arch *archs;
-    unsigned long narchs;
+    uint32_t narchs;
     struct stat stat_buf;
 
 	progname = argv[0];
@@ -260,8 +264,9 @@ char *envp[])
 		    for(j = 0; j < narch_flags; j++){
 			if(arch_flags[j].cputype ==
 				arch_flags[narch_flags].cputype &&
-			   arch_flags[j].cpusubtype ==
-				arch_flags[narch_flags].cpusubtype &&
+			   (arch_flags[j].cpusubtype & ~CPU_SUBTYPE_MASK) ==
+				(arch_flags[narch_flags].cpusubtype &
+				 ~CPU_SUBTYPE_MASK) &&
 			   strcmp(arch_flags[j].name,
 				arch_flags[narch_flags].name) == 0)
 			    break;
@@ -294,8 +299,9 @@ char *envp[])
 		for(j = 0; j < narch_flags; j++){
 		    if(arch_flags[j].cputype ==
 			    arch_flags[narch_flags].cputype &&
-		       arch_flags[j].cpusubtype ==
-			    arch_flags[narch_flags].cpusubtype &&
+		       (arch_flags[j].cpusubtype & ~CPU_SUBTYPE_MASK) ==
+			    (arch_flags[narch_flags].cpusubtype & 
+			     ~CPU_SUBTYPE_MASK) &&
 		       strcmp(arch_flags[j].name,
 			    arch_flags[narch_flags].name) == 0)
 			break;
@@ -346,7 +352,7 @@ char *envp[])
 	if(stat(input_file, &stat_buf) == -1)
 	    system_error("can't stat input file: %s", input_file);
 	writeout(archs, narchs, output_file, stat_buf.st_mode & 0777, TRUE,
-		 FALSE, FALSE, NULL);
+		 FALSE, FALSE, FALSE, NULL);
 
 	if(errors)
 	    return(EXIT_FAILURE);
@@ -383,7 +389,8 @@ enum bool nflag)
 {
 	FILE *list;
 	char buf[BUFSIZ], *symbol_name, *_symbol_name, *membername;
-	long i, len, symbol_number;
+	int32_t i, symbol_number;
+	size_t len;
 	struct indr_object *io;
 
 	/*
@@ -410,7 +417,7 @@ enum bool nflag)
 	    _symbol_name = makestr("_", symbol_name, NULL);
 
 	    if(nflag == FALSE){
-		sprintf(buf, "%05ld", symbol_number++);
+		sprintf(buf, "%05d", symbol_number++);
 		membername = makestr("INDR", buf, ".o", NULL);
 		io = enter_object(membername, symbol_name, _symbol_name,
 				  &indr_list);
@@ -433,18 +440,20 @@ static
 void
 translate_input(
 struct arch *archs,
-unsigned long narchs,
+uint32_t narchs,
 struct arch_flag *arch_flags,
-unsigned long narch_flags,
+uint32_t narch_flags,
 enum bool all_archs,
 enum bool nflag)
 {
-    unsigned long i, j, offset, size;
+    uint32_t i, j, offset, size;
     cpu_type_t cputype;
     cpu_subtype_t cpusubtype;
     struct arch_flag host_arch_flag;
     enum bool arch_process, any_processing, *arch_flag_processed;
     char *list_filename;
+    struct ar_hdr h;
+    char size_buf[sizeof(h.ar_size) + 1];
 
 	arch_flag_processed = NULL;
 	/*
@@ -464,15 +473,19 @@ enum bool nflag)
 	    if(archs[i].type == OFILE_ARCHIVE){
 		for(j = 0; j < archs[i].nmembers; j++){
 		    if(archs[i].members[j].type == OFILE_Mach_O){
-			cputype = archs[i].members[j].object->mh->cputype;
-			cpusubtype = archs[i].members[j].object->mh->cpusubtype;
+			cputype = archs[i].members[j].object->mh_cputype;
+			cpusubtype = archs[i].members[j].object->mh_cpusubtype;
 			break;
 		    }
 		}
 	    }
 	    else if(archs[i].type == OFILE_Mach_O){
-		cputype = archs[i].object->mh->cputype;
-		cpusubtype = archs[i].object->mh->cpusubtype;
+		cputype = archs[i].object->mh_cputype;
+		cpusubtype = archs[i].object->mh_cpusubtype;
+	    }
+	    else if(archs[i].fat_arch64 != NULL){
+		cputype = archs[i].fat_arch64->cputype;
+		cpusubtype = archs[i].fat_arch64->cpusubtype;
 	    }
 	    else if(archs[i].fat_arch != NULL){
 		cputype = archs[i].fat_arch->cputype;
@@ -486,7 +499,8 @@ enum bool nflag)
 	    else if(narch_flags != 0){
 		for(j = 0; j < narch_flags; j++){
 		    if(arch_flags[j].cputype == cputype &&
-		       arch_flags[j].cpusubtype == cpusubtype){
+		       (arch_flags[j].cpusubtype & ~CPU_SUBTYPE_MASK) ==
+		       (cpusubtype & ~CPU_SUBTYPE_MASK)){
 			arch_process = TRUE;
 			arch_flag_processed[j] = TRUE;
 			if(list_filenames != NULL)
@@ -498,7 +512,8 @@ enum bool nflag)
 	    else{
 		(void)get_arch_from_host(&host_arch_flag, NULL);
 		if(host_arch_flag.cputype == cputype &&
-		   host_arch_flag.cpusubtype == cpusubtype)
+		   (host_arch_flag.cpusubtype & ~CPU_SUBTYPE_MASK) ==
+		   (cpusubtype & ~CPU_SUBTYPE_MASK))
 		    arch_process = TRUE;
 	    }
 	    if(narchs != 1 && arch_process == FALSE)
@@ -534,23 +549,23 @@ enum bool nflag)
 		    archs[i].members[j].offset = offset;
 		    size = 0;
 		    if(archs[i].members[j].member_long_name == TRUE){
-			size = round(archs[i].members[j].member_name_size,
-				     sizeof(long));
+			size = rnd32(archs[i].members[j].member_name_size,
+				     sizeof(int32_t));
 			archs[i].toc_long_name = TRUE;
 		    }
 		    if(archs[i].members[j].object != NULL){
 			size += archs[i].members[j].object->object_size
 			   - archs[i].members[j].object->input_sym_info_size
 			   + archs[i].members[j].object->output_sym_info_size;
-			sprintf(archs[i].members[j].ar_hdr->ar_size, "%-*ld",
-			       (int)sizeof(archs[i].members[j].ar_hdr->ar_size),
-			       (long)(size));
+			sprintf(size_buf, "%-*ld",
+			   (int)sizeof(archs[i].members[j].ar_hdr->ar_size),
+			   (long)(size));
 			/*
 			 * This has to be done by hand because sprintf puts a
 			 * null at the end of the buffer.
 			 */
-			memcpy(archs[i].members[j].ar_hdr->ar_fmag, ARFMAG,
-			      (int)sizeof(archs[i].members[j].ar_hdr->ar_fmag));
+			memcpy(archs[i].members[j].ar_hdr->ar_size, size_buf,
+			   (int)sizeof(archs[i].members[j].ar_hdr->ar_size));
 		    }
 		    else{
 			size += archs[i].members[j].unknown_size;
@@ -592,20 +607,20 @@ struct member *member,
 struct object *object)
 {
 
-    unsigned long i, j;
+    uint32_t i, j;
 
     enum byte_sex host_byte_sex;
     struct nlist *symbols, *nlistp;
-    unsigned long nsyms, offset;
+    uint32_t nsyms, offset;
     char *strings;
-    unsigned long strings_size;
+    uint32_t strings_size;
     struct symbol *sp;
 
-    long new_nsyms;
+    int32_t new_nsyms;
     struct nlist *new_symbols;
 
-	if(object->mh->filetype == MH_DYLIB ||
-	   object->mh->filetype == MH_DYLIB_STUB){
+	if(object->mh_filetype == MH_DYLIB ||
+	   object->mh_filetype == MH_DYLIB_STUB){
 	    if(member != NULL)
 		fatal_arch(arch, member, "is a dynamic library which is not "
 			   "allowed as a member of an archive");
@@ -631,7 +646,7 @@ struct object *object)
 				      object->st->strsize;
 	if(object->dyst != NULL){
 	    object->input_sym_info_size +=
-		    object->dyst->nindirectsyms * sizeof(unsigned long);
+		    object->dyst->nindirectsyms * sizeof(uint32_t);
 	}
 
 	/*
@@ -646,7 +661,7 @@ struct object *object)
 	    if(nlistp->n_type & N_EXT){
 		if(nlistp->n_un.n_strx){
 		    if(nlistp->n_un.n_strx > 0 &&
-		       (unsigned long)nlistp->n_un.n_strx < strings_size){
+		       (uint32_t)nlistp->n_un.n_strx < strings_size){
 			sp = lookup_symbol(strings + nlistp->n_un.n_strx);
 			if(sp != NULL){
 			    if(sp->type == N_UNDF)
@@ -662,18 +677,18 @@ struct object *object)
 		    }
 		    else
 			fatal_arch(arch, member, "bad string table "
-				    "index in symbol %lu in: ", i);
+				    "index in symbol %u in: ", i);
 		}
 	    }
 	    else{
 		if(nlistp->n_un.n_strx){
 		    if(nlistp->n_un.n_strx > 0 && nlistp->n_un.n_strx <
-							    (long)strings_size)
+							(int32_t)strings_size)
 			nlistp->n_un.n_strx = add_to_string_table(
 				strings + nlistp->n_un.n_strx);
 		    else
 			fatal_arch(arch, member, "bad string table "
-				    "index in symbol %lu in: ", i);
+				    "index in symbol %u in: ", i);
 		}
 	    }
 	    nlistp++;
@@ -731,7 +746,7 @@ struct object *object)
 		string_table.index;
 	if(object->dyst != NULL){
 	    object->output_sym_info_size +=
-		    object->dyst->nindirectsyms * sizeof(unsigned long);
+		    object->dyst->nindirectsyms * sizeof(uint32_t);
 	}
 
 	if(object->seg_linkedit != NULL){
@@ -744,7 +759,7 @@ struct object *object)
 	    object->st->nsyms = nsyms;
 	    object->st->strsize = string_table.index;
 
-	    offset = ULONG_MAX;
+	    offset = UINT_MAX;
 	    if(object->st->nsyms != 0 &&
 	       object->st->symoff < offset)
 		offset = object->st->symoff;
@@ -763,11 +778,11 @@ struct object *object)
 		object->st->symoff = 0;
 
 	    if(object->dyst->nindirectsyms != 0){
-		object->output_indirect_symtab = (unsigned long *)
+		object->output_indirect_symtab = (uint32_t *)
 		    (object->object_addr + object->dyst->indirectsymoff);
 		object->dyst->indirectsymoff = offset;
 		offset += object->dyst->nindirectsyms *
-			  sizeof(unsigned long);
+			  sizeof(uint32_t);
 	    }
 	    else
 		object->dyst->indirectsymoff = 0;;
@@ -797,39 +812,40 @@ struct arch *arch,
 struct object *object)
 {
     enum byte_sex host_byte_sex;
-    unsigned long i, inew_syms, inew_undefsyms, inew_mods, indr_iextdefsym;
-    unsigned long new_ext_strsize, len, offset;
-    unsigned long *map;
+    uint32_t i, inew_syms, inew_undefsyms, inew_mods, indr_iextdefsym;
+    uint32_t new_ext_strsize, offset;
+    uint32_t *map;
+    size_t len;
     struct symbol *sp;
     char *p, *q;
     struct scattered_relocation_info *sreloc;
 
     struct nlist *symbols;
-    unsigned long nsyms;
+    uint32_t nsyms;
     char *strings;
-    unsigned long strsize;
+    uint32_t strsize;
     struct dylib_table_of_contents *tocs;
-    unsigned long ntoc;
+    uint32_t ntoc;
     struct dylib_module *mods;
-    unsigned long nmodtab;
+    uint32_t nmodtab;
     struct dylib_reference *refs;
-    unsigned long nextrefsyms;
+    uint32_t nextrefsyms;
     struct relocation_info *ext_relocs;
-    unsigned long *indirect_symtab;
+    uint32_t *indirect_symtab;
 
     struct nlist *new_symbols;
-    unsigned long new_nsyms;
+    uint32_t new_nsyms;
     char *new_strings;
-    unsigned long new_strsize;
-    unsigned long new_nlocalsym;
-    unsigned long new_nextdefsym;
-    unsigned long new_nundefsym;
+    uint32_t new_strsize;
+    uint32_t new_nlocalsym;
+    uint32_t new_nextdefsym;
+    uint32_t new_nundefsym;
     struct dylib_table_of_contents *new_tocs;
-    unsigned long new_ntoc;
+    uint32_t new_ntoc;
     struct dylib_module *new_mods;
-    unsigned long new_nmodtab;
+    uint32_t new_nmodtab;
     struct dylib_reference *new_refs;
-    unsigned long new_nextrefsyms;
+    uint32_t new_nextrefsyms;
 
 
 	/*
@@ -850,7 +866,7 @@ struct object *object)
 	nextrefsyms = object->dyst->nextrefsyms;
 	ext_relocs = (struct relocation_info *)
 		      (object->object_addr + object->dyst->extreloff);
-	indirect_symtab = (unsigned long *)
+	indirect_symtab = (uint32_t *)
 			   (object->object_addr + object->dyst->indirectsymoff);
 	/*
 	 * Swap them if needed.
@@ -881,17 +897,16 @@ struct object *object)
 	 * the symbols can be renamed.
 	 */
 	new_nsyms = 0;
-	new_strsize = sizeof(long);
+	new_strsize = sizeof(int32_t);
 	new_nlocalsym = 0;
 	new_nextdefsym = 0;
 	new_nundefsym = 0;
 	new_ext_strsize = 0;
 	for(i = 0; i < nsyms; i++){
 	    if(symbols[i].n_un.n_strx != 0){
-		if(symbols[i].n_un.n_strx < 0 ||
-		   (unsigned long)symbols[i].n_un.n_strx > strsize){
+		if((uint32_t)symbols[i].n_un.n_strx > strsize){
 		    error_arch(arch, NULL, "bad string index for symbol "
-			       "table entry %ld in: ", i);
+			       "table entry %d in: ", i);
 		    return;
 		}
 	    }
@@ -899,7 +914,7 @@ struct object *object)
 		if(symbols[i].n_value != 0){
 		    if(symbols[i].n_value > strsize){
 			error_arch(arch, NULL, "bad string index for "
-				   "indirect symbol table entry %ld in: ", i);
+				   "indirect symbol table entry %d in: ", i);
 			return;
 		    }
 		}
@@ -967,7 +982,7 @@ struct object *object)
 	    if(mods[i].module_name == 0 ||
 	       mods[i].module_name > strsize){
 		error_arch(arch, NULL, "bad string index for module_name "
-			   "of module table entry %ld in: ", i);
+			   "of module table entry %d in: ", i);
 		return;
 	    }
 	    len = strlen(strings + mods[i].module_name) + 1;
@@ -1005,14 +1020,14 @@ struct object *object)
 	 * Second pass, create the new tables.
 	 */
 	new_symbols =(struct nlist *)allocate(new_nsyms * sizeof(struct nlist));
-	new_strsize = round(new_strsize, sizeof(long));
+	new_strsize = rnd32(new_strsize, sizeof(int32_t));
 	new_strings = (char *)allocate(new_strsize);
 	new_strings[new_strsize - 3] = '\0';
 	new_strings[new_strsize - 2] = '\0';
 	new_strings[new_strsize - 1] = '\0';
 
-	memset(new_strings, '\0', sizeof(long));
-	p = new_strings + sizeof(long);
+	memset(new_strings, '\0', sizeof(int32_t));
+	p = new_strings + sizeof(int32_t);
 	q = p + new_ext_strsize;
 
 	/*
@@ -1025,15 +1040,16 @@ struct object *object)
 	 *	external strings
 	 *	local strings
 	 */
-	map = (unsigned long *)allocate(nsyms * sizeof(unsigned long));
-	memset(map, '\0', nsyms * sizeof(unsigned long));
+	map = (uint32_t *)allocate(nsyms * sizeof(uint32_t));
+	memset(map, '\0', nsyms * sizeof(uint32_t));
 	inew_syms = 0;
 	for(i = 0; i < nsyms; i++){ /* loop for local symbols */
 	    if((symbols[i].n_type & N_EXT) == 0){
 		new_symbols[inew_syms] = symbols[i];
 		if(symbols[i].n_un.n_strx != 0){
 		    strcpy(q, strings + symbols[i].n_un.n_strx);
-		    new_symbols[inew_syms].n_un.n_strx = q - new_strings;
+		    new_symbols[inew_syms].n_un.n_strx =
+			(uint32_t)(q - new_strings);
 		    q += strlen(q) + 1;
 		}
 		map[i] = inew_syms;
@@ -1052,13 +1068,15 @@ struct object *object)
 			strcpy(p, sp->indr);
 		    else
 			strcpy(p, strings + symbols[i].n_un.n_strx);
-		    new_symbols[inew_syms].n_un.n_strx = p - new_strings;
+		    new_symbols[inew_syms].n_un.n_strx =
+			(uint32_t)(p - new_strings);
 		    p += strlen(p) + 1;
 		}
 		if((symbols[i].n_type & N_TYPE) == N_INDR){
 		    if(symbols[i].n_value != 0){
 			strcpy(p, strings + symbols[i].n_value);
-			new_symbols[inew_syms].n_value = p - new_strings;
+			new_symbols[inew_syms].n_value =
+			    (uint32_t)(p - new_strings);
 			p += strlen(p) + 1;
 		    }
 		}
@@ -1069,7 +1087,7 @@ struct object *object)
 	indr_iextdefsym = inew_syms;
 	for(i = 0; i < indr_list.used; i++){ /* loop for new defined symbols*/
 	    strcpy(p, indr_list.list[i]->indr);
-	    new_symbols[inew_syms].n_un.n_strx = p - new_strings;
+	    new_symbols[inew_syms].n_un.n_strx = (uint32_t)(p - new_strings);
 	    p += strlen(p) + 1;
 	    new_symbols[inew_syms].n_type = N_INDR | N_EXT;
 	    new_symbols[inew_syms].n_desc = 0;
@@ -1077,7 +1095,7 @@ struct object *object)
 	    /* Note this name is used below for the undefined */
 	    strcpy(p, indr_list.list[i]->undef);
 	    indr_list.list[i]->undef = p;
-	    new_symbols[inew_syms].n_value = p - new_strings;
+	    new_symbols[inew_syms].n_value = (uint32_t)(p - new_strings);
 	    p += strlen(p) + 1;
 	    inew_syms++;
 	}
@@ -1093,7 +1111,7 @@ struct object *object)
 	    if(indr_list.list[i]->existing_symbol == FALSE){
 		/* Note this name is used from above for the undefined */
 		undef_map[inew_undefsyms].symbol.n_un.n_strx =
-		    indr_list.list[i]->undef - new_strings;
+		    (uint32_t)(indr_list.list[i]->undef - new_strings);
 		undef_map[inew_undefsyms].symbol.n_type = N_UNDF | N_EXT;
 		undef_map[inew_undefsyms].symbol.n_desc = 0;
 		undef_map[inew_undefsyms].symbol.n_sect = NO_SECT;
@@ -1115,7 +1133,7 @@ struct object *object)
 		    else
 			strcpy(p, strings + symbols[i].n_un.n_strx);
 		    undef_map[inew_undefsyms].symbol.n_un.n_strx =
-			p - new_strings;
+			(uint32_t)(p - new_strings);
 		    p += strlen(p) + 1;
 		}
 		undef_map[inew_undefsyms].old_symbol = TRUE;
@@ -1149,14 +1167,14 @@ struct object *object)
 	for(i = 0; i < nmodtab; i++){
 	    new_mods[inew_mods] = mods[i];
 	    strcpy(p, strings + mods[i].module_name);
-	    new_mods[inew_mods].module_name = p - new_strings;
+	    new_mods[inew_mods].module_name = (uint32_t)(p - new_strings);
 	    p += strlen(p) + 1;
 	    inew_mods++;
 	}
 	for(i = 0; i < indr_list.used; i++){
 	    memset(new_mods + inew_mods, '\0', sizeof(struct dylib_module));
 	    strcpy(p, indr_list.list[i]->membername);
-	    new_mods[inew_mods].module_name = p - new_strings;
+	    new_mods[inew_mods].module_name = (uint32_t)(p - new_strings);
 	    p += strlen(p) + 1;
 	    new_mods[inew_mods].iextdefsym = indr_iextdefsym + i;
 	    new_mods[inew_mods].nextdefsym = 1;
@@ -1216,21 +1234,21 @@ struct object *object)
 	       ext_relocs[i].r_extern == 1){
 		if(ext_relocs[i].r_symbolnum > nsyms){
 		    fatal_arch(arch, NULL, "bad r_symbolnum for external "
-			"relocation entry %ld in: ", i);
+			"relocation entry %d in: ", i);
 		}
 		ext_relocs[i].r_symbolnum = map[ext_relocs[i].r_symbolnum];
 	    }
 	    else{
 		fatal_arch(arch, NULL, "bad external relocation entry "
-		    "%ld (not external) in: ", i);
+		    "%d (not external) in: ", i);
 	    }
 	    if((ext_relocs[i].r_address & R_SCATTERED) == 0){
-		if(reloc_has_pair(object->mh->cputype, ext_relocs[i].r_type))
+		if(reloc_has_pair(object->mh_cputype, ext_relocs[i].r_type))
 		    i++;
 	    }
 	    else{
 		sreloc = (struct scattered_relocation_info *)ext_relocs + i;
-		if(reloc_has_pair(object->mh->cputype, sreloc->r_type))
+		if(reloc_has_pair(object->mh_cputype, sreloc->r_type))
 		    i++;
 	    }
 	}
@@ -1242,7 +1260,7 @@ struct object *object)
 	    if(indirect_symtab[i] != INDIRECT_SYMBOL_LOCAL &&
 	       indirect_symtab[i] != INDIRECT_SYMBOL_ABS){
 		if(indirect_symtab[i] > nsyms)
-		    fatal_arch(arch, NULL, "indirect symbol table entry %ld "
+		    fatal_arch(arch, NULL, "indirect symbol table entry %d "
 			"(past the end of the symbol table) in: ", i);
 		indirect_symtab[i] = map[indirect_symtab[i]];
 	    }
@@ -1255,7 +1273,7 @@ struct object *object)
 	object->input_sym_info_size =
 	    object->dyst->nlocrel * sizeof(struct relocation_info) +
 	    object->dyst->nextrel * sizeof(struct relocation_info) +
-	    object->dyst->nindirectsyms * sizeof(unsigned long) +
+	    object->dyst->nindirectsyms * sizeof(uint32_t) +
 	    nsyms * sizeof(struct nlist) +
 	    strsize +
 	    ntoc * sizeof(struct dylib_table_of_contents)+
@@ -1265,13 +1283,46 @@ struct object *object)
 	object->output_sym_info_size =
 	    object->dyst->nlocrel * sizeof(struct relocation_info) +
 	    object->dyst->nextrel * sizeof(struct relocation_info) +
-	    object->dyst->nindirectsyms * sizeof(unsigned long) +
+	    object->dyst->nindirectsyms * sizeof(uint32_t) +
 	    new_nsyms * sizeof(struct nlist) +
 	    new_strsize +
 	    new_ntoc * sizeof(struct dylib_table_of_contents)+
 	    new_nmodtab * sizeof(struct dylib_module) +
 	    new_nextrefsyms * sizeof(struct dylib_reference);
 
+	if(object->split_info_cmd != NULL){
+	    object->input_sym_info_size += object->split_info_cmd->datasize;
+	    object->output_sym_info_size += object->split_info_cmd->datasize;
+	}
+
+	if(object->func_starts_info_cmd != NULL){
+	    object->input_sym_info_size +=
+		object->func_starts_info_cmd->datasize;
+	    object->output_sym_info_size +=
+		object->func_starts_info_cmd->datasize;
+	}
+
+	if(object->data_in_code_cmd != NULL){
+	    object->input_sym_info_size +=
+		object->data_in_code_cmd->datasize;
+	    object->output_sym_info_size +=
+		object->data_in_code_cmd->datasize;
+	}
+
+	if(object->code_sign_drs_cmd != NULL){
+	    object->input_sym_info_size +=
+		object->code_sign_drs_cmd->datasize;
+	    object->output_sym_info_size +=
+		object->code_sign_drs_cmd->datasize;
+	}
+
+	if(object->link_opt_hint_cmd != NULL){
+	    object->input_sym_info_size +=
+		object->link_opt_hint_cmd->datasize;
+	    object->output_sym_info_size +=
+		object->link_opt_hint_cmd->datasize;
+	}
+
 	if(object->hints_cmd != NULL){ 
 	    object->input_sym_info_size +=
 		object->hints_cmd->nhints * sizeof(struct twolevel_hint);
@@ -1279,6 +1330,16 @@ struct object *object)
 		object->hints_cmd->nhints * sizeof(struct twolevel_hint);
 	}
 
+	if(object->code_sig_cmd != NULL){
+	    object->input_sym_info_size =
+		rnd32(object->input_sym_info_size, 16);
+	    object->input_sym_info_size +=
+		object->code_sig_cmd->datasize;
+	    object->output_sym_info_size =
+		rnd32(object->output_sym_info_size, 16);
+	    object->output_sym_info_size +=
+		object->code_sig_cmd->datasize;
+	}
 
 	if(object->seg_linkedit != NULL){
 	    object->seg_linkedit->filesize += object->output_sym_info_size -
@@ -1298,12 +1359,48 @@ struct object *object)
 	object->output_nextrefsyms = new_nextrefsyms;
 	object->output_loc_relocs = (struct relocation_info *)
 	    (object->object_addr + object->dyst->locreloff);
+	if(object->split_info_cmd != NULL){
+	    object->output_split_info_data = 
+	    (object->object_addr + object->split_info_cmd->dataoff);
+	    object->output_split_info_data_size = 
+		object->split_info_cmd->datasize;
+	}
+	if(object->func_starts_info_cmd != NULL){
+	    object->output_func_start_info_data = 
+	    (object->object_addr + object->func_starts_info_cmd->dataoff);
+	    object->output_func_start_info_data_size = 
+		object->func_starts_info_cmd->datasize;
+	}
+	if(object->data_in_code_cmd != NULL){
+	    object->output_data_in_code_info_data = 
+	    (object->object_addr + object->data_in_code_cmd->dataoff);
+	    object->output_data_in_code_info_data_size = 
+		object->data_in_code_cmd->datasize;
+	}
+	if(object->code_sign_drs_cmd != NULL){
+	    object->output_code_sign_drs_info_data = 
+	    (object->object_addr + object->code_sign_drs_cmd->dataoff);
+	    object->output_code_sign_drs_info_data_size = 
+		object->code_sign_drs_cmd->datasize;
+	}
+	if(object->link_opt_hint_cmd != NULL){
+	    object->output_link_opt_hint_info_data = 
+	    (object->object_addr + object->link_opt_hint_cmd->dataoff);
+	    object->output_link_opt_hint_info_data_size = 
+		object->link_opt_hint_cmd->datasize;
+	}
 	object->output_ext_relocs = ext_relocs;
 	object->output_indirect_symtab = indirect_symtab;
 	if(object->hints_cmd != NULL){
 	    object->output_hints = (struct twolevel_hint *)
 		(object->object_addr + object->hints_cmd->offset);
 	}
+	if(object->code_sig_cmd != NULL){
+	    object->output_code_sig_data = object->object_addr +
+		object->code_sig_cmd->dataoff;
+	    object->output_code_sig_data_size = 
+		object->code_sig_cmd->datasize;
+	}
 
 	if(object->object_byte_sex != host_byte_sex){
 	    /* the symbol table gets swapped by writeout() */
@@ -1337,6 +1434,26 @@ struct object *object)
 	    object->dyst->locreloff = offset;
 	    offset += object->dyst->nlocrel * sizeof(struct relocation_info);
 	}
+	if(object->split_info_cmd != NULL){
+	    object->split_info_cmd->dataoff = offset;
+	    offset += object->split_info_cmd->datasize;
+	}
+	if(object->func_starts_info_cmd != NULL){
+	    object->func_starts_info_cmd->dataoff = offset;
+	    offset += object->func_starts_info_cmd->datasize;
+	}
+	if(object->data_in_code_cmd != NULL){
+	    object->data_in_code_cmd->dataoff = offset;
+	    offset += object->data_in_code_cmd->datasize;
+	}
+	if(object->code_sign_drs_cmd != NULL){
+	    object->code_sign_drs_cmd->dataoff = offset;
+	    offset += object->code_sign_drs_cmd->datasize;
+	}
+	if(object->link_opt_hint_cmd != NULL){
+	    object->link_opt_hint_cmd->dataoff = offset;
+	    offset += object->link_opt_hint_cmd->datasize;
+	}
 	if(object->st->nsyms != 0){
 	    object->st->symoff = offset;
 	    offset += object->st->nsyms * sizeof(struct nlist);
@@ -1354,7 +1471,7 @@ struct object *object)
 	}
 	if(object->dyst->nindirectsyms != 0){
 	    object->dyst->indirectsymoff = offset;
-	    offset += object->dyst->nindirectsyms * sizeof(unsigned long);
+	    offset += object->dyst->nindirectsyms * sizeof(uint32_t);
 	}
 	if(object->dyst->ntoc != 0){
 	    object->dyst->tocoff = offset;
@@ -1374,6 +1491,11 @@ struct object *object)
 	    object->st->stroff = offset;
 	    offset += object->st->strsize;
 	}
+	if(object->code_sig_cmd != NULL){
+	    offset = rnd32(offset, 16);
+	    object->code_sig_cmd->dataoff = offset;
+	    offset += object->code_sig_cmd->datasize;
+	}
 }
 
 /*
@@ -1414,18 +1536,18 @@ void
 make_indr_objects(
 struct arch *arch)
 {
-    unsigned long i;
+    int32_t i;
     cpu_type_t cputype;
     cpu_subtype_t cpusubtype;
     enum byte_sex target_byte_sex;
     struct indr_object *io;
     struct nlist *output_symbols, *undef, *indr;
     char *output_strings, *object_addr;
-    unsigned long object_size;
+    uint32_t object_size;
     struct mach_header *mh;
     struct symtab_command *st;
     struct ar_hdr *ar_hdr;
-    unsigned long indr_time, size;
+    uint32_t indr_time, size;
     unsigned short indr_mode;
     int oumask;
     gid_t gid;
@@ -1446,8 +1568,8 @@ struct arch *arch)
 	for(i = arch->nmembers - 1; i >= 0 ;i--){
 	    if(arch->members[i].object != NULL){
 		if(cputype == 0){
-		    cputype = arch->members[i].object->mh->cputype;
-		    cpusubtype = arch->members[i].object->mh->cpusubtype;
+		    cputype = arch->members[i].object->mh_cputype;
+		    cpusubtype = arch->members[i].object->mh_cpusubtype;
 		}
 		if(target_byte_sex == UNKNOWN_BYTE_SEX){
 		    target_byte_sex = arch->members[i].object->object_byte_sex;
@@ -1465,7 +1587,7 @@ struct arch *arch)
 	 * Now loop through the indr list creating the objects for each indirect
 	 * symbol to be added to this archive.
 	 */
-	indr_time = time(0);
+	indr_time = (uint32_t)time(0);
 	oumask = umask(0);
 	indr_mode = S_IFREG | (0666 & ~oumask);
 	(void)umask(oumask);
@@ -1572,11 +1694,11 @@ static
 void
 enter_symbol(
 char *name,
-long type,
+int32_t type,
 char *indr,
 struct indr_object *io)
 {
-    long hash_key;
+    int32_t hash_key;
     struct symbol *sp;
 
 	hash_key = hash_string(name) % SYMBOL_HASH_SIZE;
@@ -1608,7 +1730,7 @@ struct symbol *
 lookup_symbol(
 char *name)
 {
-    long hash_key;
+    int32_t hash_key;
     struct symbol *sp;
 
 	hash_key = hash_string(name) % SYMBOL_HASH_SIZE;
@@ -1664,8 +1786,8 @@ start_string_table()
 	    string_table.size = INITIAL_STRING_TABLE_SIZE;
 	    string_table.strings = (char *)allocate(string_table.size);
 	}
-	memset(string_table.strings, '\0', sizeof(long));
-	string_table.index = sizeof(long);
+	memset(string_table.strings, '\0', sizeof(int32_t));
+	string_table.index = sizeof(int32_t);
 }
 
 /*
@@ -1673,11 +1795,12 @@ start_string_table()
  * returns the index of the string in the table.
  */
 static
-long
+int32_t
 add_to_string_table(
 char *p)
 {
-    long len, index;
+    size_t len;
+    uint32_t index;
 
 	len = strlen(p) + 1;
 	if(string_table.size < string_table.index + len){
@@ -1699,9 +1822,9 @@ static
 void
 end_string_table()
 {
-    unsigned long length;
+    uint32_t length;
 
-	length = round(string_table.index, sizeof(unsigned long));
+	length = rnd32(string_table.index, sizeof(uint32_t));
 	memset(string_table.strings + string_table.index, '\0',
 	       length - string_table.index);
 	string_table.index = length;
