@@ -542,6 +542,9 @@ void *cookie)
 	process_flags.nlibs = 0;
 	process_flags.lib_names = NULL;
 
+	llvm_bundle_pointer = NULL;
+	llvm_bundle_size = 0;
+
 	if(ofile->mh == NULL && ofile->mh64 == NULL){
 #ifdef LTO_SUPPORT
 	    if(ofile->lto != NULL)
@@ -1767,6 +1770,11 @@ char *arch_name)
 	       (symbols[i].nl.n_desc & N_ALT_ENTRY) == N_ALT_ENTRY)
 		    printf("[alt entry] ");
 
+	    if(ofile->mh_filetype == MH_OBJECT &&
+	       ((symbols[i].nl.n_type & N_TYPE) != N_UNDF) &&
+	       (symbols[i].nl.n_desc & N_COLD_FUNC) == N_COLD_FUNC)
+		    printf("[cold func] ");
+
 	    if((symbols[i].nl.n_desc & N_ARM_THUMB_DEF) == N_ARM_THUMB_DEF)
 		    printf("[Thumb] ");
 
