@@ -70,11 +70,24 @@ enum mach_o_part_type {
     MP_INDIRECT_SYMBOL_TABLE,
     MP_EXT_RELOCS,
     MP_LOC_RELOCS,
+    MP_SPLIT_INFO,
     MP_SYMBOL_TABLE,
     MP_HINTS_TABLE,
     MP_STRING_TABLE,
     MP_EXT_STRING_TABLE,
     MP_LOC_STRING_TABLE,
+    MP_CODE_SIG,
+    MP_FUNCTION_STARTS,
+    MP_DATA_IN_CODE,
+    MP_CODE_SIGN_DRS,
+    MP_LINK_OPT_HINT,
+    MP_DYLD_CHAINED_FIXUPS,
+    MP_DYLD_EXPORTS_TRIE,
+    MP_DYLD_INFO_REBASE,
+    MP_DYLD_INFO_BIND,
+    MP_DYLD_INFO_WEAK_BIND,
+    MP_DYLD_INFO_LAZY_BIND,
+    MP_DYLD_INFO_EXPORT,
     MP_EMPTY_SPACE
 };
 static char *mach_o_part_type_names[] = {
@@ -92,11 +105,24 @@ static char *mach_o_part_type_names[] = {
     "MP_INDIRECT_SYMBOL_TABLE",
     "MP_EXT_RELOCS",
     "MP_LOC_RELOCS",
+    "MP_SPLIT_INFO",
     "MP_SYMBOL_TABLE",
     "MP_HINTS_TABLE",
     "MP_STRING_TABLE",
     "MP_EXT_STRING_TABLE",
     "MP_LOC_STRING_TABLE",
+    "MP_CODE_SIG",
+    "MP_FUNCTION_STARTS",
+    "MP_DATA_IN_CODE",
+    "MP_CODE_SIGN_DRS",
+    "MP_LINK_OPT_HINT",
+    "MP_DYLD_CHAINED_FIXUPS",
+    "MP_DYLD_EXPORTS_TRIE",
+    "MP_DYLD_INFO_REBASE",
+    "MP_DYLD_INFO_BIND",
+    "MP_DYLD_INFO_WEAK_BIND",
+    "MP_DYLD_INFO_LAZY_BIND",
+    "MP_DYLD_INFO_EXPORT",
     "MP_EMPTY_SPACE"
 };
 
@@ -116,9 +142,17 @@ char *progname = NULL;
 /* The ofile for the Mach-O file argument to the program */
 static struct ofile ofile;
 
+/* The -arch flag if any, and its offset and size in the file */
+static struct arch_flag *arch_flag = NULL;
+uint64_t arch_offset = 0;
+uint64_t arch_size = 0;
+enum bool arch_found = FALSE;
+
 static struct nlist *sorted_symbols = NULL;
 static struct nlist_64 *sorted_symbols64 = NULL;
 
+static void usage(void);
+
 static void create_file_parts(
     char *file_name);
 static struct file_part *new_file_part(
@@ -139,7 +173,7 @@ static void print_mach_o_parts(
     struct mach_o_part *mp);
 
 static void print_parts_for_page(
-    unsigned long page_number);
+    uint64_t page_number);
 static void print_arch(
     struct file_part *fp);
 static void print_file_part(
@@ -161,10 +195,14 @@ static int compare64(
     struct nlist_64 *p1,
     struct nlist_64 *p2);
 
+/* apple_version is created by the libstuff/Makefile */
+extern char apple_version[];
+char *version = apple_version;
+
 /*
  * pagestuff is invoked as follows:
  *
- *	% pagestuff mach-o pagenumber [pagenumber ...]
+ *	% pagestuff mach-o [-arch name] [-p] [-a] pagenumber [pagenumber ...]
  *
  * It prints out what stuff is on the page numbers listed.
  */
@@ -174,18 +212,39 @@ int argc,
 char *argv[])
 {
     int i, start;
-    unsigned long j, page_number;
+    uint64_t j, page_number;
     char *endp;
+    struct arch_flag a;
 
 	progname = argv[0];
-	if(argc < 3){
-	    fprintf(stderr, "Usage: %s mach-o pagenumber [pagenumber ...]\n",
-		    progname);
-	    exit(EXIT_FAILURE);
-	}
+	if(argc < 3)
+	    usage();
 	start = 2;
 
+	if(strcmp(argv[start], "-arch") == 0){
+	    if(start + 1 == argc){
+		error("missing argument to -arch option");
+		exit(EXIT_FAILURE);
+	    }
+	    if(get_arch_from_flag(argv[start+1], &a) == 0){
+		error("unknown architecture specification flag: "
+		      "%s %s", argv[start], argv[start+1]);
+		exit(EXIT_FAILURE);
+	    }
+	    arch_flag = &a;
+	    start += 2;
+	    if(argc < 5)
+		usage();
+        }
+
 	create_file_parts(argv[1]);
+	if(arch_flag != NULL && arch_found == FALSE){
+	    error("file: %s does not contain architecture: %s", argv[1],
+		  arch_flag->name);
+	    exit(EXIT_FAILURE);
+	}
+	if(errors)
+	    exit(EXIT_FAILURE);
 	if(strcmp(argv[start], "-p") == 0){
 	    print_file_parts();
 	    start++;
@@ -196,7 +255,12 @@ char *argv[])
 	    exit(EXIT_SUCCESS);
 
 	if(strcmp(argv[start], "-a") == 0){
-	    page_number = (ofile.file_size + vm_page_size - 1) / vm_page_size;
+	    if(arch_flag == NULL)
+		page_number = (ofile.file_size + vm_page_size - 1) /
+			      vm_page_size;
+	    else
+		page_number = (arch_size + vm_page_size - 1) /
+			      vm_page_size;
 	    for(j = 0; j < page_number; j++){
 		print_parts_for_page(j);
 	    }
@@ -216,6 +280,16 @@ char *argv[])
 	return(EXIT_SUCCESS);
 }
 
+static
+void
+usage(void)
+{
+	fprintf(stderr, "Usage: %s mach-o [-arch name] [-p] [-a] "
+		"pagenumber [pagenumber ...]\n",
+		progname);
+	exit(EXIT_FAILURE);
+}
+
 static
 void
 create_file_parts(
@@ -223,7 +297,7 @@ char *file_name)
 {
     static struct file_part *fp;
 
-	if(ofile_map(file_name, NULL, NULL, &ofile, FALSE) == FALSE)
+	if(ofile_map(file_name, arch_flag, NULL, &ofile, FALSE) == FALSE)
 	    exit(EXIT_FAILURE);
 
 	/* first create an empty space for the whole file */
@@ -236,8 +310,13 @@ char *file_name)
 	if(ofile.file_type == OFILE_FAT){
 	    fp = new_file_part();
 	    fp->offset = 0;
-	    fp->size = sizeof(struct fat_header) +
-		       ofile.fat_header->nfat_arch * sizeof(struct fat_arch);
+	    fp->size = sizeof(struct fat_header);
+	    if(ofile.fat_header->magic == FAT_MAGIC_64)
+		fp->size += ofile.fat_header->nfat_arch *
+			    sizeof(struct fat_arch_64);
+	    else
+		fp->size += ofile.fat_header->nfat_arch *
+			    sizeof(struct fat_arch);
 	    fp->type = FP_FAT_HEADERS;
 	    insert_file_part(fp);
 
@@ -249,10 +328,31 @@ char *file_name)
 			progname);
 		}
 		else if(ofile.arch_type == OFILE_Mach_O){
+		    if(arch_flag != NULL && arch_found == FALSE &&
+		       arch_flag->cputype == ofile.mh_cputype &&
+		       (arch_flag->cpusubtype & ~CPU_SUBTYPE_MASK) ==
+			(ofile.mh_cpusubtype & ~CPU_SUBTYPE_MASK)){
+			if(ofile.fat_header->magic == FAT_MAGIC_64){
+			    arch_offset =
+				ofile.fat_archs64[ofile.narch].offset; 
+			    arch_size = ofile.fat_archs64[ofile.narch].size; 
+			}
+			else{
+			    arch_offset = ofile.fat_archs[ofile.narch].offset; 
+			    arch_size = ofile.fat_archs[ofile.narch].size; 
+			}
+			arch_found = TRUE;
+		    }
 		    /* make mach-o parts for this */
 		    fp = new_file_part();
-		    fp->offset = ofile.fat_archs[ofile.narch].offset;
-		    fp->size = ofile.fat_archs[ofile.narch].size;
+		    if(ofile.fat_header->magic == FAT_MAGIC_64){
+			fp->offset = ofile.fat_archs64[ofile.narch].offset;
+			fp->size = ofile.fat_archs64[ofile.narch].size;
+		    }
+		    else{
+			fp->offset = ofile.fat_archs[ofile.narch].offset;
+			fp->size = ofile.fat_archs[ofile.narch].size;
+		    }
 		    fp->type = FP_MACH_O;
 		    insert_file_part(fp);
 		    create_mach_o_parts(fp);
@@ -269,6 +369,14 @@ char *file_name)
 		progname);
 	}
 	else if(ofile.file_type == OFILE_Mach_O){
+	    if(arch_flag != NULL && arch_found == FALSE &&
+	       arch_flag->cputype == ofile.mh_cputype &&
+	       (arch_flag->cpusubtype & ~CPU_SUBTYPE_MASK) ==
+		(ofile.mh_cpusubtype & ~CPU_SUBTYPE_MASK)){
+		arch_offset = 0;
+		arch_size = ofile.file_size;
+		arch_found = TRUE;
+	    }
 	    /* make mach-o parts for this */
 	    fp = new_file_part();
 	    fp->offset = 0;
@@ -376,9 +484,11 @@ void)
 	prev = NULL;
 	offset = 0;
 	for(p = file_parts; p != NULL; p = p->next){
-	    printf("%s\n", file_part_type_names[p->type]);
-	    printf("    offset = %llu\n", p->offset);
-	    printf("    size = %llu\n", p->size);
+	    if(arch_flag == NULL){
+		printf("%s\n", file_part_type_names[p->type]);
+		printf("    offset = %llu\n", p->offset);
+		printf("    size = %llu\n", p->size);
+	    }
 	    if(prev != NULL)
 		if(prev != p->prev)
 		    printf("bad prev pointer\n");
@@ -386,8 +496,10 @@ void)
 	    if(offset != p->offset)
 		    printf("bad offset\n");
 	    offset += p->size;
-	    if(p->type == FP_MACH_O)
-		print_mach_o_parts(p->mp);
+	    if(p->type == FP_MACH_O){
+		if(arch_flag == NULL || p->offset == arch_offset)
+		    print_mach_o_parts(p->mp);
+	    }
 	}
 }
 
@@ -396,12 +508,13 @@ void
 create_mach_o_parts(
 struct file_part *fp)
 {
-    unsigned long i, j;
+    uint32_t i, j;
     uint32_t ncmds, filetype;
     struct mach_o_part *mp;
     struct load_command *lc;
     struct symtab_command *st;
     struct dysymtab_command *dyst;
+    struct dyld_info_command *dyld_info;
     struct twolevel_hints_command *hints;
     struct segment_command *sg;
     struct segment_command_64 *sg64;
@@ -409,10 +522,15 @@ struct file_part *fp)
     struct section_64 *s64;
     struct nlist *allocated_symbols, *symbols;
     struct nlist_64 *allocated_symbols64, *symbols64;
-    unsigned long ext_low, ext_high, local_low, local_high, n_strx, n_type;
+    uint32_t ext_low, ext_high, local_low, local_high, n_strx, n_type;
     char *strings;
     struct dylib_module *modtab;
     struct dylib_module_64 *modtab64;
+    struct linkedit_data_command *split_info, *code_sig, *func_starts,
+			         *data_in_code, *code_sign_drs;
+    struct linkedit_data_command *link_opt_hint;
+    struct linkedit_data_command *dyld_chained_fixups, *dyld_exports_trie;
+    enum bool dylib_stub;
 
 	mp = new_mach_o_part();
 	mp->offset = fp->offset;
@@ -427,12 +545,14 @@ struct file_part *fp)
 	    fp->mh64 = NULL;
 	    mp->size = sizeof(struct mach_header) + ofile.mh->sizeofcmds;
 	    ncmds = ofile.mh->ncmds;
+	    dylib_stub = ofile.mh->filetype == MH_DYLIB_STUB;
 	}
 	else{
 	    fp->mh64 = ofile.mh64;
 	    fp->mh = NULL;
 	    mp->size = sizeof(struct mach_header_64) + ofile.mh64->sizeofcmds;
 	    ncmds = ofile.mh64->ncmds;
+	    dylib_stub = ofile.mh64->filetype == MH_DYLIB_STUB;
 	}
 	mp->type = MP_MACH_HEADERS;
 	insert_mach_o_part(fp, mp);
@@ -440,10 +560,20 @@ struct file_part *fp)
 
 	st = NULL;
 	dyst = NULL;
+	dyld_info = NULL;
 	hints = NULL;
 	symbols = NULL;
 	symbols64 = NULL;
 	strings = NULL;
+	split_info = NULL;
+	code_sig = NULL;
+	func_starts = NULL;
+	data_in_code = NULL;
+	code_sign_drs = NULL;
+	link_opt_hint = NULL;
+	dyld_chained_fixups = NULL;
+	dyld_exports_trie = NULL;
+
 	lc = ofile.load_commands;
 	for(i = 0; i < ncmds; i++){
 	    if(st == NULL && lc->cmd == LC_SYMTAB){
@@ -452,10 +582,40 @@ struct file_part *fp)
 	    else if(dyst == NULL && lc->cmd == LC_DYSYMTAB){
 		dyst = (struct dysymtab_command *)lc;
 	    }
+	    else if(dyld_info == NULL && lc->cmd == LC_DYLD_INFO_ONLY){
+		dyld_info = (struct dyld_info_command *)lc;
+	    }
 	    else if(hints == NULL && lc->cmd == LC_TWOLEVEL_HINTS){
 		hints = (struct twolevel_hints_command *)lc;
 	    }
-	    else if(lc->cmd == LC_SEGMENT){
+	    else if(split_info == NULL && lc->cmd == LC_SEGMENT_SPLIT_INFO){
+		split_info = (struct linkedit_data_command *)lc;
+	    }
+	    else if(code_sig == NULL && lc->cmd == LC_CODE_SIGNATURE){
+		code_sig = (struct linkedit_data_command *)lc;
+	    }
+	    else if(func_starts == NULL && lc->cmd == LC_FUNCTION_STARTS){
+		func_starts = (struct linkedit_data_command *)lc;
+	    }
+	    else if(data_in_code == NULL && lc->cmd == LC_DATA_IN_CODE){
+		data_in_code = (struct linkedit_data_command *)lc;
+	    }
+	    else if(code_sign_drs == NULL && lc->cmd == LC_DYLIB_CODE_SIGN_DRS){
+		code_sign_drs = (struct linkedit_data_command *)lc;
+	    }
+	    else if(link_opt_hint == NULL &&
+		    lc->cmd == LC_LINKER_OPTIMIZATION_HINT){
+		link_opt_hint = (struct linkedit_data_command *)lc;
+	    }
+	    else if (dyld_chained_fixups == NULL &&
+		     lc->cmd == LC_DYLD_CHAINED_FIXUPS) {
+		dyld_chained_fixups = (struct linkedit_data_command*)lc;
+	    }
+	    else if (dyld_exports_trie == NULL &&
+		     lc->cmd == LC_DYLD_EXPORTS_TRIE) {
+		dyld_exports_trie = (struct linkedit_data_command*)lc;
+	    }
+	    else if(lc->cmd == LC_SEGMENT && dylib_stub == FALSE){
 		sg = (struct segment_command *)lc;
 		s = (struct section *)
 		      ((char *)sg + sizeof(struct segment_command));
@@ -469,7 +629,9 @@ struct file_part *fp)
 			mp->s64 = NULL;
 			insert_mach_o_part(fp, mp);
 		    }
-		    if((s->flags & SECTION_TYPE) != S_ZEROFILL && s->size != 0){
+		    if((s->flags & SECTION_TYPE) != S_ZEROFILL &&
+		       (s->flags & SECTION_TYPE) != S_THREAD_LOCAL_ZEROFILL &&
+		       s->size != 0){
 			mp = new_mach_o_part();
 			mp->offset = fp->offset + s->offset;
 			mp->size = s->size;
@@ -478,7 +640,9 @@ struct file_part *fp)
 			mp->s64 = NULL;
 			insert_mach_o_part(fp, mp);
 		    }
-		    if((s->flags & SECTION_TYPE) == S_ZEROFILL && s->size != 0){
+		    if(((s->flags & SECTION_TYPE) == S_ZEROFILL ||
+			(s->flags & SECTION_TYPE) == S_THREAD_LOCAL_ZEROFILL) &&
+		       s->size != 0){
 			if(s->addr - sg->vmaddr < sg->filesize){
 			    mp = new_mach_o_part();
 			    mp->offset = fp->offset + sg->fileoff +
@@ -496,7 +660,7 @@ struct file_part *fp)
 		    s++;
 		}
 	    }
-	    else if(lc->cmd == LC_SEGMENT_64){
+	    else if(lc->cmd == LC_SEGMENT_64 && dylib_stub == FALSE){
 		sg64 = (struct segment_command_64 *)lc;
 		s64 = (struct section_64 *)
 		      ((char *)sg64 + sizeof(struct segment_command_64));
@@ -511,6 +675,7 @@ struct file_part *fp)
 			insert_mach_o_part(fp, mp);
 		    }
 		    if((s64->flags & SECTION_TYPE) != S_ZEROFILL &&
+		       (s64->flags & SECTION_TYPE) != S_THREAD_LOCAL_ZEROFILL &&
 		       s64->size != 0){
 			mp = new_mach_o_part();
 			mp->offset = fp->offset + s64->offset;
@@ -520,7 +685,9 @@ struct file_part *fp)
 			mp->s = NULL;
 			insert_mach_o_part(fp, mp);
 		    }
-		    if((s64->flags & SECTION_TYPE) == S_ZEROFILL && 
+		    if(((s64->flags & SECTION_TYPE) == S_ZEROFILL ||
+			(s64->flags & SECTION_TYPE) ==
+						    S_THREAD_LOCAL_ZEROFILL) && 
 		       s64->size != 0){
 			if(s64->addr - sg64->vmaddr < sg64->filesize){
 			    mp = new_mach_o_part();
@@ -633,6 +800,13 @@ struct file_part *fp)
 		mp->type = MP_HINTS_TABLE;
 		insert_mach_o_part(fp, mp);
 	    }
+	    if(split_info != NULL && split_info->datasize != 0){
+		mp = new_mach_o_part();
+		mp->offset = fp->offset + split_info->dataoff;
+		mp->size = split_info->datasize;
+		mp->type = MP_SPLIT_INFO;
+		insert_mach_o_part(fp, mp);
+	    }
 	    if(dyst->ntoc != 0){
 		mp = new_mach_o_part();
 		mp->offset = fp->offset + dyst->tocoff;
@@ -660,7 +834,7 @@ struct file_part *fp)
 	    if(dyst->nindirectsyms != 0){
 		mp = new_mach_o_part();
 		mp->offset = fp->offset + dyst->indirectsymoff;
-		mp->size = dyst->nindirectsyms * sizeof(unsigned long);
+		mp->size = dyst->nindirectsyms * sizeof(uint32_t);
 		mp->type = MP_INDIRECT_SYMBOL_TABLE;
 		insert_mach_o_part(fp, mp);
 	    }
@@ -702,6 +876,18 @@ struct file_part *fp)
 		    }
 		    if(n_strx == 0)
 			continue;
+		    /*
+		     * If this object has any bad string indexes don't try to
+		     * break the string table into the local and external
+		     * parts.
+		     */
+		    if(n_strx > st->strsize){
+			ext_low = st->strsize;
+			local_low = st->strsize;
+			ext_high = 0;
+			local_high = 0;
+			break;
+		    }
 		    if(n_type & N_EXT ||
 		       (filetype == MH_EXECUTE && n_type & N_PEXT)){
 			if(n_strx > ext_high)
@@ -744,7 +930,7 @@ struct file_part *fp)
 		    }
 		}
 
-		if(ext_high < local_low){
+		if(ext_high < local_low && local_low < local_high){
 		    mp = new_mach_o_part();
 		    mp->offset = fp->offset + st->stroff + ext_low;
 		    mp->size = ext_high - ext_low +
@@ -805,6 +991,92 @@ struct file_part *fp)
 		insert_mach_o_part(fp, mp);
 	    }
 	}
+	if(dyld_info != NULL){
+	    if(dyld_info->rebase_size != 0){
+		mp = new_mach_o_part();
+		mp->offset = fp->offset + dyld_info->rebase_off;
+		mp->size = dyld_info->rebase_size;
+		mp->type = MP_DYLD_INFO_REBASE;
+		insert_mach_o_part(fp, mp);
+	    }
+	    if(dyld_info->bind_size != 0){
+		mp = new_mach_o_part();
+		mp->offset = fp->offset + dyld_info->bind_off;
+		mp->size = dyld_info->bind_size;
+		mp->type = MP_DYLD_INFO_BIND;
+		insert_mach_o_part(fp, mp);
+	    }
+	    if(dyld_info->weak_bind_size != 0){
+		mp = new_mach_o_part();
+		mp->offset = fp->offset + dyld_info->weak_bind_off;
+		mp->size = dyld_info->weak_bind_size;
+		mp->type = MP_DYLD_INFO_WEAK_BIND;
+		insert_mach_o_part(fp, mp);
+	    }
+	    if(dyld_info->lazy_bind_size != 0){
+		mp = new_mach_o_part();
+		mp->offset = fp->offset + dyld_info->lazy_bind_off;
+		mp->size = dyld_info->lazy_bind_size;
+		mp->type = MP_DYLD_INFO_LAZY_BIND;
+		insert_mach_o_part(fp, mp);
+	    }
+	    if(dyld_info->export_size != 0){
+		mp = new_mach_o_part();
+		mp->offset = fp->offset + dyld_info->export_off;
+		mp->size = dyld_info->export_size;
+		mp->type = MP_DYLD_INFO_EXPORT;
+		insert_mach_o_part(fp, mp);
+	    }
+	}
+	if(code_sig != NULL && code_sig->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + code_sig->dataoff;
+	    mp->size = code_sig->datasize;
+	    mp->type = MP_CODE_SIG;
+	    insert_mach_o_part(fp, mp);
+	}
+	if(func_starts != NULL && func_starts->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + func_starts->dataoff;
+	    mp->size = func_starts->datasize;
+	    mp->type = MP_FUNCTION_STARTS;
+	    insert_mach_o_part(fp, mp);
+	}
+	if(data_in_code != NULL && data_in_code->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + data_in_code->dataoff;
+	    mp->size = data_in_code->datasize;
+	    mp->type = MP_DATA_IN_CODE;
+	    insert_mach_o_part(fp, mp);
+	}
+	if(code_sign_drs != NULL && code_sign_drs->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + code_sign_drs->dataoff;
+	    mp->size = code_sign_drs->datasize;
+	    mp->type = MP_CODE_SIGN_DRS;
+	    insert_mach_o_part(fp, mp);
+	}
+	if(link_opt_hint != NULL && link_opt_hint->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + link_opt_hint->dataoff;
+	    mp->size = link_opt_hint->datasize;
+	    mp->type = MP_LINK_OPT_HINT;
+	    insert_mach_o_part(fp, mp);
+	}
+	if(dyld_chained_fixups != NULL && dyld_chained_fixups->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + dyld_chained_fixups->dataoff;
+	    mp->size = dyld_chained_fixups->datasize;
+	    mp->type = MP_DYLD_CHAINED_FIXUPS;
+	    insert_mach_o_part(fp, mp);
+	}
+	if(dyld_exports_trie != NULL && dyld_exports_trie->datasize != 0){
+	    mp = new_mach_o_part();
+	    mp->offset = fp->offset + dyld_exports_trie->dataoff;
+	    mp->size = dyld_exports_trie->datasize;
+	    mp->type = MP_DYLD_EXPORTS_TRIE;
+	    insert_mach_o_part(fp, mp);
+	}
 }
 
 static
@@ -896,23 +1168,28 @@ print_mach_o_parts(
 struct mach_o_part *mp)
 {
     struct mach_o_part *p, *prev;
-    unsigned long offset;
+    uint64_t offset;
+    char *indent;
 
+	if(arch_flag == NULL)
+	    indent = "    ";
+	else
+	    indent = "";
 	offset = 0;
 	prev = NULL;
 	if(mp != NULL)
 	    offset = mp->offset;
 	for(p = mp; p != NULL; p = p->next){
 	    if(p->type == MP_SECTION)
-		printf("    MP_SECTION (%.16s,%.16s)\n",
+		printf("%sMP_SECTION (%.16s,%.16s)\n", indent,
 		p->s->segname, p->s->sectname);
 	    else if(p->type == MP_SECTION_64)
-		printf("    MP_SECTION_64 (%.16s,%.16s)\n",
+		printf("%sMP_SECTION_64 (%.16s,%.16s)\n", indent,
 		p->s64->segname, p->s64->sectname);
 	    else
-		printf("    %s\n", mach_o_part_type_names[p->type]);
-	    printf("\toffset = %llu\n", p->offset);
-	    printf("\tsize = %llu\n", p->size);
+		printf("%s%s\n", indent, mach_o_part_type_names[p->type]);
+	    printf("%s    offset = %llu\n", indent, p->offset - arch_offset);
+	    printf("%s    size = %llu\n", indent, p->size);
 	    if(prev != NULL)
 		if(prev != p->prev)
 		    printf("bad prev pointer\n");
@@ -926,24 +1203,38 @@ struct mach_o_part *mp)
 static
 void
 print_parts_for_page(
-unsigned long page_number)
+uint64_t page_number)
 {
     uint64_t offset, size, low_addr, high_addr, new_low_addr, new_high_addr;
     struct file_part *fp;
     struct mach_o_part *mp;
     enum bool printed;
     enum bool sections, sections64;
+    const char *arch_name;
 
 	offset = page_number * vm_page_size;
 	size = vm_page_size;
 	low_addr = 0;
 	high_addr = 0;
 
-	if(offset > ofile.file_size){
-	    printf("File has no page %lu (file has only %lu pages)\n",
-		   page_number,
-		   (ofile.file_size + vm_page_size - 1) / vm_page_size);
-	    return;
+	if(arch_flag == NULL){
+	    if(offset > ofile.file_size){
+		printf("File has no page %llu (file has only %u pages)\n",
+		       page_number, (uint32_t)((ofile.file_size +
+					        vm_page_size -1) /
+					      vm_page_size));
+	        return;
+	    }
+	}
+	else{
+	    if(offset > arch_size){
+		printf("File for architecture %s has no page %llu (has only %u "
+		       "pages)\n", arch_flag->name,
+		       page_number, (uint32_t)((arch_size +
+					        vm_page_size -1) /
+					      vm_page_size));
+	        return;
+	    }
 	}
 
 	/*
@@ -952,13 +1243,13 @@ unsigned long page_number)
 	 */
 	printed = FALSE;
 	for(fp = file_parts; fp != NULL; fp = fp->next){
-	    if(offset + size <= fp->offset)
+	    if(offset + size <= fp->offset - arch_offset)
 		continue;
-	    if(offset > fp->offset + fp->size)
+	    if(offset > fp->offset - arch_offset + fp->size)
 		continue;
 	    switch(fp->type){
 	    case FP_FAT_HEADERS:
-		printf("File Page %lu contains fat file headers\n",
+		printf("File Page %llu contains fat file headers\n",
 		       page_number);
 		printed = TRUE;
 		break;
@@ -966,32 +1257,33 @@ unsigned long page_number)
 		sections = FALSE;
 		sections64 = FALSE;
 		for(mp = fp->mp; mp != NULL; mp = mp->next){
-		    if(offset + size <= mp->offset)
+		    if(offset + size <= mp->offset - arch_offset)
 			continue;
-		    if(offset > mp->offset + mp->size)
+		    if(offset > mp->offset - arch_offset + mp->size)
 			continue;
 		    switch(mp->type){
 		    case MP_MACH_HEADERS:
-			printf("File Page %lu contains Mach-O headers",
+			printf("File Page %llu contains Mach-O headers",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_SECTION:
-			printf("File Page %lu contains contents of "
+			printf("File Page %llu contains contents of "
 			       "section (%.16s,%.16s)", page_number,
 			       mp->s->segname, mp->s->sectname);
 			print_arch(fp);
 			printed = TRUE;
-			if(offset < mp->offset)
+			if(offset < mp->offset - arch_offset)
 			    new_low_addr = mp->s->addr;
 			else
-			    new_low_addr = mp->s->addr + offset - mp->offset;
-			if(offset + size > mp->offset + mp->size)
+			    new_low_addr = mp->s->addr + offset - mp->offset - 
+					   arch_offset;
+			if(offset + size > mp->offset - arch_offset + mp->size)
 			    new_high_addr = mp->s->addr + mp->s->size;
 			else
 			    new_high_addr = mp->s->addr +
-				(offset + size - mp->offset);
+				(offset + size - (mp->offset - arch_offset));
 			if(sections == FALSE){
 			    low_addr = new_low_addr;
 			    high_addr = new_high_addr;
@@ -1005,20 +1297,22 @@ unsigned long page_number)
 			sections = TRUE;
 			break;
 		    case MP_SECTION_64:
-			printf("File Page %lu contains contents of "
+			printf("File Page %llu contains contents of "
 			       "section (%.16s,%.16s)", page_number,
 			       mp->s64->segname, mp->s64->sectname);
 			print_arch(fp);
 			printed = TRUE;
-			if(offset < mp->offset)
+			if(offset < mp->offset - arch_offset)
 			    new_low_addr = mp->s64->addr;
 			else
-			    new_low_addr = mp->s64->addr + offset - mp->offset;
-			if(offset + size > mp->offset + mp->size)
+			    new_low_addr = mp->s64->addr + offset -
+					   (mp->offset - arch_offset);
+			if(offset + size > (mp->offset - arch_offset) +
+					   mp->size)
 			    new_high_addr = mp->s64->addr + mp->s64->size;
 			else
 			    new_high_addr = mp->s64->addr +
-				(offset + size - mp->offset);
+				(offset + size - (mp->offset - arch_offset));
 			if(sections64 == FALSE){
 			    low_addr = new_low_addr;
 			    high_addr = new_high_addr;
@@ -1032,109 +1326,187 @@ unsigned long page_number)
 			sections64 = TRUE;
 			break;
 		    case MP_RELOCS:
-			printf("File Page %lu contains relocation entries for "
+			printf("File Page %llu contains relocation entries for "
 			       "section (%.16s,%.16s)", page_number,
 			       mp->s->segname, mp->s->sectname);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_RELOCS_64:
-			printf("File Page %lu contains relocation entries for "
+			printf("File Page %llu contains relocation entries for "
 			       "section (%.16s,%.16s)", page_number,
 			       mp->s64->segname, mp->s64->sectname);
 			print_arch(fp);
 			printed = TRUE;
 			break;
+		    case MP_SPLIT_INFO:
+			printf("File Page %llu contains local of info to split "
+			       "segments", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
 		    case MP_LOCAL_SYMBOLS:
-			printf("File Page %lu contains symbol table for "	
+			printf("File Page %llu contains symbol table for "
 			       "non-global symbols", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_EXTDEF_SYMBOLS:
-			printf("File Page %lu contains symbol table for "	
+			printf("File Page %llu contains symbol table for "
 			       "defined global symbols", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_UNDEF_SYMBOLS:
-			printf("File Page %lu contains symbol table for "	
+			printf("File Page %llu contains symbol table for "
 			       "undefined symbols", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_TOC:
-			printf("File Page %lu contains table of contents",
+			printf("File Page %llu contains table of contents",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_MODULE_TABLE:
-			printf("File Page %lu contains module table",
+			printf("File Page %llu contains module table",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_REFERENCE_TABLE:
-			printf("File Page %lu contains reference table",
+			printf("File Page %llu contains reference table",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_INDIRECT_SYMBOL_TABLE:
-			printf("File Page %lu contains indirect symbols table",
+			printf("File Page %llu contains indirect symbols table",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_EXT_RELOCS:
-			printf("File Page %lu contains external relocation "
+			printf("File Page %llu contains external relocation "
 			       "entries", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_LOC_RELOCS:
-			printf("File Page %lu contains local relocation "
+			printf("File Page %llu contains local relocation "
 			       "entries", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_SYMBOL_TABLE:
-			printf("File Page %lu contains symbol table",
+			printf("File Page %llu contains symbol table",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_HINTS_TABLE:
-			printf("File Page %lu contains hints table",
+			printf("File Page %llu contains hints table",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_STRING_TABLE:
-			printf("File Page %lu contains string table",
+			printf("File Page %llu contains string table",
 			       page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_EXT_STRING_TABLE:
-			printf("File Page %lu contains string table for "
+			printf("File Page %llu contains string table for "
 			       "external symbols", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
 		    case MP_LOC_STRING_TABLE:
-			printf("File Page %lu contains string table for "
+			printf("File Page %llu contains string table for "
 			       "local symbols", page_number);
 			print_arch(fp);
 			printed = TRUE;
 			break;
+		    case MP_CODE_SIG:
+			printf("File Page %llu contains data of code signature",
+			       page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_FUNCTION_STARTS:
+			printf("File Page %llu contains data of function starts",
+			       page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DATA_IN_CODE:
+			printf("File Page %llu contains info for data in code",
+			       page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_CODE_SIGN_DRS:
+			printf("File Page %llu contains info for code signing "
+			       "DRs copied from linked dylib", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_LINK_OPT_HINT:
+			printf("File Page %llu contains info for linker "
+			       "optimization hints", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_CHAINED_FIXUPS:
+			printf("File Page %llu contains dyld info for chained "
+			       "fixups", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_EXPORTS_TRIE:
+			printf("File Page %llu contains dyld info for exported "
+			       "symbols (TRIE)", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_INFO_REBASE:
+			printf("File Page %llu contains dyld info for sliding "
+			       "an image", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_INFO_BIND:
+			printf("File Page %llu contains dyld info for binding "
+			       "symbols", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_INFO_WEAK_BIND:
+			printf("File Page %llu contains dyld info for weak bound "
+			       "symbols", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_INFO_LAZY_BIND:
+			printf("File Page %llu contains dyld info for lazy bound "
+			       "symbols", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
+		    case MP_DYLD_INFO_EXPORT:
+			printf("File Page %llu contains dyld info for symbols "
+			       "exported by a dylib", page_number);
+			print_arch(fp);
+			printed = TRUE;
+			break;
 		    case MP_EMPTY_SPACE:
 			break;
 		    }
 		}
 		if(sections == TRUE || sections64 == TRUE){
-		    printf("Symbols on file page %lu virtual address 0x%llx to "
+		    printf("Symbols on file page %llu virtual address 0x%llx to "
 			   "0x%llx\n", page_number, low_addr, high_addr);
 		    if(sections == TRUE)
 			print_symbols(fp, low_addr, high_addr);
@@ -1161,14 +1533,18 @@ unsigned long page_number)
 		continue;
 	    if(fp->type == FP_MACH_O){
 		for(mp = fp->mp; mp != NULL; mp = mp->next){
-		    if(offset + size <= mp->offset)
+		    if(offset + size <= mp->offset - arch_offset)
 			continue;
-		    if(offset > mp->offset + mp->size)
+		    if(offset > (mp->offset - arch_offset) + mp->size)
 			continue;
-		    printf("File Page %lu contains empty space in the Mach-O "
-			   "file for %s between:\n", page_number,
-			   get_arch_name_from_types(fp->mh->cputype,
-						    fp->mh->cpusubtype));
+		    if(fp->mh != NULL)
+			arch_name = get_arch_name_from_types(fp->mh->cputype,
+						    	fp->mh->cpusubtype);
+		    else
+			arch_name = get_arch_name_from_types(fp->mh64->cputype,
+						    	fp->mh64->cpusubtype);
+		    printf("File Page %llu contains empty space in the Mach-O "
+			   "file for %s between:\n", page_number, arch_name);
 		    if(mp->prev == NULL)
 			printf("    the start of the Mach-O file");
 		    else{
@@ -1187,7 +1563,7 @@ unsigned long page_number)
 		}
 		break;
 	    }
-	    printf("File Page %lu contains empty space in the file between:\n",
+	    printf("File Page %llu contains empty space in the file between:\n",
 		   page_number);
 	    if(fp->prev == NULL)
 		printf("    the start of the file");
@@ -1231,14 +1607,20 @@ void
 print_file_part(
 struct file_part *fp)
 {
+    const char *arch_name;
+
 	switch(fp->type){
 	case FP_FAT_HEADERS:
 	    printf("fat file headers");
 	    break;
 	case FP_MACH_O:
-	    printf("Mach-O file for %s",
-		   get_arch_name_from_types(fp->mh->cputype,
-					    fp->mh->cpusubtype));
+	    if(fp->mh != NULL)
+		arch_name = get_arch_name_from_types(fp->mh->cputype,
+						fp->mh->cpusubtype);
+	    else
+		arch_name = get_arch_name_from_types(fp->mh64->cputype,
+						fp->mh64->cpusubtype);
+	    printf("Mach-O file for %s", arch_name);
 	    break;
 	case FP_EMPTY_SPACE:
 	    printf("empty space");
@@ -1271,6 +1653,9 @@ struct mach_o_part *mp)
 	    printf("relocation entries for section (%.16s,%.16s)",
 		   mp->s64->segname, mp->s64->sectname);
 	    break;
+	case MP_SPLIT_INFO:
+	    printf("local of info to split segments");
+	    break;
 	case MP_LOCAL_SYMBOLS:
 	    printf("symbol table for non-global symbols");
 	    break;
@@ -1313,6 +1698,42 @@ struct mach_o_part *mp)
 	case MP_LOC_STRING_TABLE:
 	    printf("string table for local symbols");
 	    break;
+	case MP_CODE_SIG:
+	    printf("data of code signature");
+	    break;
+	case MP_FUNCTION_STARTS:
+	    printf("data of function starts");
+	    break;
+	case MP_DATA_IN_CODE:
+	    printf("info for data in code");
+	    break;
+	case MP_CODE_SIGN_DRS:
+	    printf("info for code signing DRs copied from linked dylibs");
+	    break;
+	case MP_LINK_OPT_HINT:
+	    printf("info for linker optimization hints");
+	    break;
+	case MP_DYLD_CHAINED_FIXUPS:
+	    printf("dyld info for chained fixups");
+	    break;
+	case MP_DYLD_EXPORTS_TRIE:
+	    printf("dyld info for exported symbols (TRIE)");
+	    break;
+	case MP_DYLD_INFO_REBASE:
+	    printf("dyld info for sliding an image");
+	    break;
+	case MP_DYLD_INFO_BIND:
+	    printf("dyld info for binding symbols");
+	    break;
+	case MP_DYLD_INFO_WEAK_BIND:
+	    printf("dyld info for binding weak symbols");
+	    break;
+	case MP_DYLD_INFO_LAZY_BIND:
+	    printf("dyld info for binding lazy symbols");
+	    break;
+	case MP_DYLD_INFO_EXPORT:
+	    printf("dyld info for exported symbols");
+	    break;
 	case MP_EMPTY_SPACE:
 	    printf("empty space");
 	    break;
@@ -1326,7 +1747,7 @@ struct file_part *fp,
 uint64_t low_addr,
 uint64_t high_addr)
 {
-    unsigned long i, count;
+    uint32_t i, count;
 
 	if(fp->st == NULL)
 	    return;
@@ -1361,7 +1782,7 @@ struct file_part *fp,
 uint64_t low_addr,
 uint64_t high_addr)
 {
-    unsigned long i, count;
+    uint32_t i, count;
 
 	if(fp->st == NULL)
 	    return;
